@@ -19,6 +19,7 @@ from __future__ import with_statement
 
 import dbus
 from dbus.mainloop.glib import DBusGMainLoop
+import contextlib
 import logging
 import os
 import shutil
@@ -32,6 +33,7 @@ from ubuntuone.syncdaemon import (
     interfaces,
     volume_manager,
     main,
+    local_rescan,
 )
 from ubuntuone.syncdaemon.dbus_interface import (
     DBusInterface,
@@ -41,10 +43,24 @@ from ubuntuone.syncdaemon.dbus_interface import (
     DBUS_IFACE_AUTH_NAME,
 )
 from twisted.internet import defer
+from twisted.python import failure
 from twisted.trial.unittest import TestCase as TwistedTestCase
 from zope.interface import implements
 from oauth import oauth
 
+
+@contextlib.contextmanager
+def environ(env_var, new_value):
+    """context manager to replace/add an environ value"""
+    old_value = os.environ.get(env_var, None)
+    os.environ[env_var] = new_value
+    yield
+    if old_value is None:
+        os.environ.pop(env_var)
+    else:
+        os.environ[env_var] = old_value
+
+
 class FakeOAuthClient(object):
     """ Fake OAuthClient"""
 
@@ -105,53 +121,62 @@ class FakeMain(main.Main):
                                                self.partials_dir, self.vm)
         self.event_q = event_queue.EventQueue(self.fs)
         self.fs.register_eq(self.event_q)
-        self.action_q = FakeActionQueue(self.event_q)
-        self.state = main.SyncDaemonStateManager(self, 2, 0)
+        self.action_q = FakeActionQueue(self.event_q, main=self)
+        self.state_manager = main.StateManager(self, 2)
         self.event_q.subscribe(self.vm)
         self.vm.init_root()
         self.hash_q = FakeHashQueue()
         self.mark = FakeMark()
         self.dbus_iface = FakeDBusInterface()
+        self.lr = local_rescan.LocalRescan(self.vm, self.fs,
+                                           self.event_q, self.action_q)
 
     def _connect_aq(self, token):
-        """ Connect the fake action queue """
+        """Connect the fake action queue."""
         self.action_q.connect()
 
     def _disconnect_aq(self):
-        """ Disconnect the fake action queue """
+        """Disconnect the fake action queue."""
         self.action_q.disconnect()
 
     def get_access_token(self):
-        """fake get token"""
+        """Fake get token."""
         return None
 
     def check_version(self):
-        """ Check the client protocol version matches that of the server. """
+        """Check the client protocol version matches that of the server."""
         self.event_q.push('SYS_PROTOCOL_VERSION_OK')
 
     def authenticate(self):
-        """ Do the OAuth dance. """
-        self.event_q.push('SYS_OAUTH_OK')
+        """Do the OAuth dance."""
+        self.event_q.push('SYS_AUTH_OK')
 
     def set_capabilities(self):
-        """Set the capabilities"""
+        """Set the capabilities."""
         self.event_q.push('SYS_SET_CAPABILITIES_OK')
 
     def get_root(self, root_mdid):
-        """ Ask que AQ for our root's uuid. """
+        """Ask que AQ for our root's uuid."""
         return defer.succeed('root_uuid')
 
     def server_rescan(self):
-        """ Do the server rescan? naaa! """
-        self.event_q.push('SYS_SERVER_RESCAN_STARTING')
+        """Do the server rescan? naaa!"""
         self.event_q.push('SYS_SERVER_RESCAN_DONE')
         return defer.succeed('root_uuid')
 
+    def local_rescan(self):
+        """Do the local rescan? naaa!"""
+        self.event_q.push('SYS_LOCAL_RESCAN_DONE')
+        return defer.succeed(True)
 
 
 class BaseTwistedTestCase(TwistedTestCase):
-    """ Base TestCase that provides:
+    """Base TestCase with helper methods to handle temp dir.
+
+    This class provides:
         mktemp(name): helper to create temporary dirs
+        rmtree(path): support read-only shares
+        makedirs(path): support read-only shares
     """
 
     def mktemp(self, name='temp'):
@@ -164,7 +189,11 @@ class BaseTwistedTestCase(TwistedTestCase):
 
     @property
     def tmpdir(self):
-        """ default tmpdir: 'module name'/'class name'"""
+        """Default tmpdir: module/class/test_method."""
+        # check if we already generated the root path
+        root_dir = getattr(self, '__root', None)
+        if root_dir:
+            return root_dir
         MAX_FILENAME = 32 # some platforms limit lengths of filenames
         base = os.path.join(self.__class__.__module__[:MAX_FILENAME],
                             self.__class__.__name__[:MAX_FILENAME],
@@ -172,22 +201,26 @@ class BaseTwistedTestCase(TwistedTestCase):
         # use _trial_temp dir, it should be os.gwtcwd()
         # define the root temp dir of the testcase, pylint: disable-msg=W0201
         self.__root = os.path.join(os.getcwd(), base)
-        return os.path.join(self.__root, 'tmpdir')
+        return self.__root
 
     def rmtree(self, path):
-        """ rmtree that handle ro parent(s) and childs. """
+        """Custom rmtree that handle ro parent(s) and childs."""
+        if not os.path.exists(path):
+            return
         # change perms to rw, so we can delete the temp dir
         if path != getattr(self, '__root', None):
             os.chmod(os.path.dirname(path), 0755)
-        os.chmod(path, 0755)
+        if not os.access(path, os.W_OK):
+            os.chmod(path, 0755)
         # pylint: disable-msg=W0612
         for dirpath, dirs, files in os.walk(path):
             for dir in dirs:
-                os.chmod(os.path.join(dirpath, dir), 0777)
+                if not os.access(os.path.join(dirpath, dir), os.W_OK):
+                    os.chmod(os.path.join(dirpath, dir), 0777)
         shutil.rmtree(path)
 
     def makedirs(self, path):
-        """ makedirs that handle ro parent. """
+        """Custom makedirs that handle ro parent."""
         parent = os.path.dirname(path)
         if os.path.exists(parent):
             os.chmod(parent, 0755)
@@ -195,23 +228,30 @@ class BaseTwistedTestCase(TwistedTestCase):
 
     def setUp(self):
         TwistedTestCase.setUp(self)
-        # invalidate the current config
-        config_file = os.path.join(self.mktemp('config'), 'syncdaemon.conf')
+        # use the config from the branch
+        self.old_get_config_files = config.get_config_files
+        config.get_config_files = lambda: [os.path.join(os.environ['ROOTDIR'],
+                                                   'data', 'syncdaemon.conf')]
         # fake a very basic config file with sane defaults for the tests
-        with open(config_file, 'w') as fp:
+        self.config_file = os.path.join(self.mktemp('config'), 'syncdaemon.conf')
+        with open(self.config_file, 'w') as fp:
             fp.write('[bandwidth_throttling]\n')
             fp.write('on = False\n')
             fp.write('read_limit = -1\n')
             fp.write('write_limit = -1\n')
+        # invalidate the current config
         config._user_config = None
-        config.get_user_config(config_file=config_file)
+        config.get_user_config(config_file=self.config_file)
 
     def tearDown(self):
         """ cleanup the temp dir. """
         # invalidate the current config
         config._user_config = None
-        config.get_user_config()
-        if hasattr(self, '__root'):
+        # restore the old get_config_files
+        config.get_config_files = self.old_get_config_files
+        self.rmtree(os.path.dirname(self.config_file))
+        root_dir = getattr(self, '__root', None)
+        if root_dir:
             self.rmtree(self.__root)
         return TwistedTestCase.tearDown(self)
 
@@ -247,8 +287,7 @@ class DBusTwistedTestCase(BaseTwistedTestCase):
         self.busName = self.dbus_iface.busName
         self.bus.set_exit_on_disconnect(False)
         self.dbus_iface.connect()
-        self.event_q.push('SYS_WAIT_FOR_LOCAL_RESCAN')
-        self.event_q.push('SYS_LOCAL_RESCAN_DONE')
+        self.event_q.push('SYS_INIT_DONE')
         self.signal_receivers = set()
         self.action_q.content_queue.set_change_notification_cb(
             self.dbus_iface.status.emit_content_queue_changed)
@@ -274,9 +313,12 @@ class DBusTwistedTestCase(BaseTwistedTestCase):
         self.log.info("finished test %s.%s", self.__class__.__name__,
                       self._testMethodName)
 
-    def error_handler(self, *error):
+    def error_handler(self, error):
         """ default error handler for DBus calls. """
-        self.fail(error)
+        if isinstance(error, failure.Failure):
+            self.fail(error.getErrorMessage())
+        else:
+            self.fail(str(error))
 
     def cleanup_signal_receivers(self, signal_receivers):
         """ cleanup self.signal_receivers and returns a deferred """
@@ -285,7 +327,8 @@ class DBusTwistedTestCase(BaseTwistedTestCase):
             d = defer.Deferred()
             def callback(*args):
                 """ callback that accepts *args. """
-                d.callback(args)
+                if not d.called:
+                    d.callback(args)
             self.bus.call_async(dbus.bus.BUS_DAEMON_NAME,
                                 dbus.bus.BUS_DAEMON_PATH,
                                 dbus.bus.BUS_DAEMON_IFACE, 'RemoveMatch', 's',
@@ -297,17 +340,14 @@ class DBusTwistedTestCase(BaseTwistedTestCase):
             return defer.succeed(True)
 
 
-
 class FakeActionQueue(object):
-    """ stub implementation """
+    """Stub implementation."""
 
     implements(interfaces.IActionQueue)
 
     def __init__(self, eq, *args, **kwargs):
         """ Creates the instance """
         self.eq = self.event_queue = eq
-        self.client = action_queue.ActionQueueProtocol()
-        self.client.disconnect = lambda: None
         self.uploading = {}
         self.downloading = {}
         # pylint: disable-msg=C0103
@@ -324,79 +364,79 @@ class FakeActionQueue(object):
         # throttling attributes
         self.readLimit = None
         self.writeLimit = None
-        self.throttling = False
+        self.throttling_enabled = False
 
     def connect(self, host=None, port=None, user_ssl=False):
-        """ stub implementation """
+        """Stub implementation."""
         self.eq.push('SYS_CONNECTION_MADE')
 
     def disconnect(self):
-        """ stub implementation """
-        pass
+        """Stub implementation."""
 
-    def enable_throttling(self, value):
-        self.throttling = value
+    def enable_throttling(self):
+        """Stub implementation."""
+        self.throttling_enabled = True
+
+    def disable_throttling(self):
+        """Stub implementation."""
+        self.throttling_enabled = False
 
     def cancel_download(self, share_id, node_id):
-        """ stub implementation """
-        pass
+        """Stub implementation."""
 
     def cancel_upload(self, share_id, node_id):
-        """ stub implementation """
-        pass
+        """Stub implementation."""
 
     def download(self, share_id, node_id, server_hash, fileobj):
-        """ stub implementation """
-        pass
+        """Stub implementation."""
 
     def upload(self, share_id, node_id, previous_hash, hash, crc32,
                size, deflated_size, fileobj):
-        """ stub implementation """
-        pass
+        """Stub implementation."""
 
     def make_file(self, share_id, parent_id, name, marker):
-        """ stub implementation """
-        pass
+        """Stub implementation."""
 
     def make_dir(self, share_id, parent_id, name, marker):
-        """ stub implementation """
-        pass
+        """Stub implementation."""
 
     def move(self, share_id, node_id, old_parent_id, new_parent_id, new_name):
-        """ stub implementation """
-        pass
+        """Stub implementation."""
 
     def unlink(self, share_id, node_id):
-        """ stub implementation """
-        pass
+        """Stub implementation."""
 
     def query(self, items):
-        """ stub implementation """
-        pass
+        """Stub implementation."""
 
     def listdir(self, share_id, node_id, server_hash, fileobj):
-        """ stub implementation """
-        pass
+        """Stub implementation."""
 
     def list_shares(self):
+        """Stub implementation."""
+
+    def list_volumes(self):
         """ stub implementation """
         pass
 
     def answer_share(self, share_id, answer):
-        """ stub implementation """
+        """Stub implementation."""
         self.eq.push('AQ_ANSWER_SHARE_OK', share_id, answer)
 
     def create_share(self, *args):
         """ sutb implementation """
-        pass
 
     def inquire_free_space(self, share_id):
-        """ stub implementation """
-        pass
+        """Stub implementation."""
 
     def inquire_account_info(self):
-        """ stub implementation """
-        pass
+        """Stub implementation."""
+
+    def delete_volume(self, volume_id):
+        """Stub implementation."""
+
+    def change_public_access(self, share_id, node_id, is_public):
+        """Stub implementation."""
 
 
 class MementoHandler(logging.Handler):
@@ -462,17 +502,19 @@ class FakeNetworkManager(DBusExposedObject):
         except Exception, e: # pylint: disable-msg=W0703
             error_handler(e)
 
+
 class FakeVolumeManager(object):
     """ A volume manager that only knows one share, the root"""
 
     def __init__(self, root_path):
         """ Creates the instance"""
-        self.root = volume_manager.Share(root_path, access_level='Modify')
+        self.root = volume_manager.Share(path=root_path, access_level='Modify')
         self.shares = {'':self.root}
+        self.udfs = {}
         self.log = logging.getLogger('ubuntuone.SyncDaemon.VM-test')
 
     def add_share(self, share):
-        """ Adss share to the shares dict """
+        """Add share to the shares dict."""
         self.shares[share.id] = share
         # if the share don't exists, create it
         if not os.path.exists(share.path):
@@ -481,10 +523,41 @@ class FakeVolumeManager(object):
         if not share.can_write():
             os.chmod(share.path, 0555)
 
-    def on_server_root(self, _):
-        """
-        Do nothing
-        """
+    def add_udf(self, udf):
+        """Add udf to the udfs dict."""
+        self.udfs[udf.id] = udf
+
+    def handle_SYS_ROOT_RECEIVED(self, root_id):
+        """Do nothing."""
+
+    def share_deleted(self, _):
+        """Do nothing."""
+
+    def get_volume(self, id):
+        """Returns a share or a UDF."""
+        try:
+            return self.shares[id]
+        except KeyError:
+            return self.udfs[id]
+
+    def get_volumes(self, access_level='Modify', all_volumes=False):
+        """Simple get_volumes for FakeVolumeManager."""
+        for share in self.shares.values():
+            if all_volumes or share.access_level == access_level:
+                yield share
+        for udf in self.udfs.values():
+            if all_volumes or udf.subscribed:
+                yield udf
+
+    def unsubscribe_udf(self, udf_id):
+        """Mark the UDF with udf_id as unsubscribed."""
+        udf = self.udfs[udf_id]
+        udf.subscribed = False
+        self.udfs[udf_id] = udf
+
+    def delete_volume(self, volume_id):
+        """Request the deletion of a volume."""
+
 
 # OAuth stubs
 class FakeLoginProcessor(LoginProcessor):
@@ -495,7 +568,8 @@ class FakeLoginProcessor(LoginProcessor):
         LoginProcessor.__init__(self, dbus_object, use_libnotify=False)
         self.next_login_cb = None
 
-    def login(self, realm, consumer_key, error_handler=None, reply_handler=None, do_login=True):
+    def login(self, realm, consumer_key, error_handler=None,
+              reply_handler=None, do_login=True):
         """Stub, call self.next_login_cb or send NewCredentials if
         self.next_login_cb isn't defined.
         """
@@ -541,3 +615,46 @@ class FakeLogin(Login):
         self.busName.get_bus().release_name(self.busName.get_name())
         self.remove_from_connection()
 
+
+class FakeLogger(object):
+    """Helper logging class."""
+    def __init__(self):
+        self.logged = dict(debug=[], warning=[], info=[])
+
+    def _log(self, log, txt, args):
+        """Really logs."""
+        if args:
+            txt = txt % args
+        log.append(txt)
+
+    def warning(self, txt, *args):
+        """WARNING logs."""
+        self._log(self.logged['warning'], txt, args)
+
+    def debug(self, txt, *args):
+        """DEBUG logs."""
+        self._log(self.logged['debug'], txt, args)
+
+    def info(self, txt, *args):
+        """INFO logs."""
+        self._log(self.logged['info'], txt, args)
+
+
+class Listener(object):
+    """Helper class to gather events."""
+
+    def __init__(self):
+        self.events = []
+
+    def handle_default(self, event_name, *args, **kwargs):
+        """Keep record of every event."""
+        self.events.append((event_name, args, kwargs))
+
+
+class DummyClass(object):
+    """Dummy class, does nothing."""
+
+    def __getattribute__(self, name):
+        """Any attribute is a no-op."""
+        return lambda *args, **kwargs: None
+
