@@ -17,7 +17,6 @@
 """tests for the syncdaemon config module """
 from __future__ import with_statement
 
-import contextlib
 import logging
 import os
 from ConfigParser import ConfigParser
@@ -32,36 +31,16 @@ from ubuntuone.syncdaemon import (
 from contrib.testing.testcase import (
     BaseTwistedTestCase,
     DBusTwistedTestCase,
+    environ,
 )
 
 
-@contextlib.contextmanager
-def environ(env_var, new_value):
-    """context manager to replace/add an environ value"""
-    old_value = os.environ.get(env_var, None)
-    os.environ[env_var] = new_value
-    yield
-    if old_value is None:
-        os.environ.pop(env_var)
-    else:
-        os.environ[env_var] = old_value
-
-
 class TestConfigBasic(BaseTwistedTestCase):
     """Basic _Config object tests"""
 
     def setUp(self):
         BaseTwistedTestCase.setUp(self)
         self.test_root = self.mktemp()
-        self.old_get_config_files = config.get_config_files
-        config.get_config_files = lambda: [os.path.join(os.environ['ROOT_DIR'],
-                                                   'data', 'syncdaemon.conf')]
-
-    def tearDown(self):
-        """cleanup everything, also the config module globals"""
-        config._user_config = None
-        config.get_config_files = self.old_get_config_files
-        BaseTwistedTestCase.tearDown(self)
 
     def assertThrottlingSection(self, expected, current, on, read, write):
         """Assert for equality two ConfigParser and against the on, read and
@@ -85,8 +64,8 @@ class TestConfigBasic(BaseTwistedTestCase):
         # create the config object with an empty config file
         conf = config._Config(conf_file)
         self.assertEquals(False, conf.get_throttling())
-        self.assertEquals(None, conf.get_throttling_read_limit())
-        self.assertEquals(None, conf.get_throttling_write_limit())
+        self.assertEquals(2097152, conf.get_throttling_read_limit())
+        self.assertEquals(2097152, conf.get_throttling_write_limit())
 
     def test_load_basic(self):
         """test loading the config file containing only the throttling values"""
@@ -206,7 +185,6 @@ class TestConfigBasic(BaseTwistedTestCase):
         conf_1.read(conf_file)
         self.assertThrottlingSection(conf_1, conf, False, 1000, 100)
 
-
     def test_load_negative_limits(self):
         """test loading the config file with negative read/write limits"""
         conf_file = os.path.join(self.test_root, 'test_load_config.conf')
@@ -233,7 +211,77 @@ class TestConfigBasic(BaseTwistedTestCase):
         conf = config._Config(conf_file)
         self.assertEquals(True, conf.get_throttling())
         self.assertEquals(1, conf.get_throttling_read_limit())
-        self.assertEquals(None, conf.get_throttling_write_limit())
+        self.assertEquals(2097152, conf.get_throttling_write_limit())
+
+    def test_override(self):
+        """test loading the config file containing only the throttling values"""
+        conf_file = os.path.join(self.test_root, 'test_load_config.conf')
+        # write some throttling values to the config file
+        with open(conf_file, 'w') as fp:
+            fp.write('[bandwidth_throttling]\n')
+            fp.write('on = True\n')
+            fp.write('read_limit = 1000\n')
+            fp.write('write_limit = 200\n')
+        conf = config._Config(conf_file)
+        conf_orig = config._Config(conf_file)
+        overridden_opts = [('bandwidth_throttling', 'on', False)]
+        conf.override_options(overridden_opts)
+        self.assertEquals(False, conf.get_throttling())
+        self.assertFalse(conf.get_throttling() == conf_orig.get_throttling())
+        self.assertEquals(1000, conf.get_throttling_read_limit())
+        self.assertEquals(200, conf.get_throttling_write_limit())
+        conf.save()
+        # load the config in a barebone ConfigParser and check
+        conf_1 = ConfigParser()
+        conf_1.read(conf_file)
+        self.assertThrottlingSection(conf_1, conf_orig, True, 1000, 200)
+
+    def test_load_udf_autosubscribe(self):
+        """Test load/set/override of udf_autosubscribe config value."""
+        conf_file = os.path.join(self.test_root, 'test_udf_autosubscribe_config.conf')
+        # write some throttling values to the config file
+        with open(conf_file, 'w') as fp:
+            fp.write('[__main__]\n')
+            fp.write('log_level = INFO\n')
+            fp.write('disable_ssl_verify = True\n')
+            fp.write('udf_autosubscribe = True\n')
+            fp.write('\n')
+            fp.write('[bandwidth_throttling]\n')
+            fp.write('on = True\n')
+            fp.write('read_limit = 1000\n')
+            fp.write('write_limit = 200\n')
+
+        # keep a original around
+        conf_orig = config._Config(conf_file)
+
+        # load the config
+        conf = config._Config(conf_file)
+        self.assertTrue(conf.get_udf_autosubscribe())
+        # change it to False
+        conf.set_udf_autosubscribe(False)
+        self.assertFalse(conf.get_udf_autosubscribe())
+        # save, load and check
+        conf.save()
+        conf_1 = config._Config(conf_file)
+        self.assertFalse(conf_1.get_udf_autosubscribe())
+        # change it to True
+        conf.set_udf_autosubscribe(True)
+        self.assertTrue(conf.get_udf_autosubscribe())
+        # save, load and check
+        conf.save()
+        conf_1 = config._Config(conf_file)
+        self.assertTrue(conf_1.get_udf_autosubscribe())
+
+        # load the config, check the override of the value
+        conf = config._Config(conf_file)
+        self.assertTrue(conf.get_udf_autosubscribe())
+        overridden_opts = [('__main__', 'udf_autosubscribe', False)]
+        conf.override_options(overridden_opts)
+        self.assertFalse(conf.get_udf_autosubscribe())
+        self.assertFalse(conf.get_udf_autosubscribe() == conf_orig.get_udf_autosubscribe())
+        conf.save()
+        conf_1 = config._Config(conf_file)
+        self.assertEquals(True, conf_1.get_udf_autosubscribe())
 
 
 class ConfigglueParsersTests(BaseTwistedTestCase):
@@ -245,11 +293,13 @@ class ConfigglueParsersTests(BaseTwistedTestCase):
         unset_value = '-1'
         bad_value = 'hola'
         invalid_value = None
+        zero_value = '0'
         parser = config.throttling_limit_parser
         self.assertEquals(20480, parser(good_value))
         self.assertEquals(None, parser(unset_value))
         self.assertRaises(ValueError, parser, bad_value)
         self.assertRaises(TypeError, parser, invalid_value)
+        self.assertEquals(None, parser(zero_value))
 
     def test_log_level_parser(self):
         """Test log_level_parser"""
@@ -297,19 +347,6 @@ class ConfigglueParsersTests(BaseTwistedTestCase):
 
 class TestDBusConfig(DBusTwistedTestCase):
 
-    def setUp(self):
-        """Fake the user_config file, and setup all SD stuff"""
-        self.old_get_config_files = config.get_config_files
-        config.get_config_files = lambda: [os.path.join(os.environ['ROOT_DIR'],
-                                                   'data', 'syncdaemon.conf')]
-        return DBusTwistedTestCase.setUp(self)
-
-    def tearDown(self):
-        """cleanup everything, also the config module globals"""
-        config._user_config = None
-        config.get_config_files = self.old_get_config_files
-        return DBusTwistedTestCase.tearDown(self)
-
     def test_get_throttling_limits(self):
         """Test getting throttling limits"""
         dbus_config = self.main.dbus_iface.config
@@ -357,3 +394,122 @@ class TestDBusConfig(DBusTwistedTestCase):
         conf = ConfigParser()
         conf.read(config.get_user_config().config_file)
         self.assertEquals(conf.get(config.THROTTLING, 'on'), 'False')
+
+    def test_files_sync_enabled(self):
+        """Test for toggling files sync."""
+        dbus_config = self.main.dbus_iface.config
+        enabled = dbus_config.files_sync_enabled()
+        self.assertTrue(enabled)
+        dbus_config.set_files_sync_enabled(False)
+        enabled = dbus_config.files_sync_enabled()
+        self.assertFalse(enabled)
+        dbus_config.set_files_sync_enabled(True)
+        enabled = dbus_config.files_sync_enabled()
+        self.assertTrue(enabled)
+        conf = ConfigParser()
+        conf.read(config.get_user_config().config_file)
+        self.assertEquals(conf.get(config.MAIN, 'files_sync_enabled'), 'True')
+
+
+class SyncDaemonConfigParserTests(BaseTwistedTestCase):
+    """Tests for SyncDaemonConfigParser"""
+
+    def setUp(self):
+        BaseTwistedTestCase.setUp(self)
+        self.test_root = self.mktemp()
+        self.default_config = os.path.join(os.environ['ROOTDIR'], 'data',
+                                           'syncdaemon.conf')
+        self.logging_config = os.path.join(os.environ['ROOTDIR'], 'data',
+                                           'logging.conf')
+        self.cp = config.SyncDaemonConfigParser()
+        self.cp.readfp(file(self.default_config))
+        self.cp.readfp(file(self.logging_config))
+
+    def test_log_level_old_config(self):
+        """Test log_level upgrade hook."""
+        conf_file = os.path.join(self.test_root, 'test_old_config.conf')
+        # write some throttling values to the config file
+        with open(conf_file, 'w') as fp:
+            fp.write('[__main__]\n')
+            fp.write('log_level = DEBUG\n')
+        self.assertTrue(os.path.exists(conf_file))
+        self.cp.read([conf_file])
+        self.cp.parse_all()
+        self.assertEquals(self.cp.get('logging', 'level').value, 10)
+
+    def test_log_level_new_config(self):
+        """Test log_level upgrade hook with new config"""
+        conf_file = os.path.join(self.test_root, 'test_new_config.conf')
+        # write some throttling values to the config file
+        with open(conf_file, 'w') as fp:
+            fp.write('[logging]\n')
+            fp.write('level = DEBUG\n')
+        self.assertTrue(os.path.exists(conf_file))
+        self.cp.read([conf_file])
+        self.cp.parse_all()
+        self.assertEquals(self.cp.get('logging', 'level').value, 10)
+
+    def test_log_level_old_and_new_config(self):
+        """Test log_level upgrade hook with a mixed config"""
+        conf_file = os.path.join(self.test_root, 'test_old_and_new_config.conf')
+        # write some throttling values to the config file
+        with open(conf_file, 'w') as fp:
+            fp.write('[__main__]\n')
+            fp.write('log_level = NOTE\n')
+            fp.write('[logging]\n')
+            fp.write('level = DEBUG\n')
+        self.assertTrue(os.path.exists(conf_file))
+        self.cp.read([conf_file])
+        self.cp.parse_all()
+        self.assertEquals(self.cp.get('logging', 'level').value, 10)
+
+    def test_old_default_config(self):
+        """Test log_level upgrade hook with an old default config"""
+        self.cp.read(config.get_config_files()[0])
+        # fake an old config
+        value = self.cp.get('logging', 'level.default')
+        help = self.cp.get('logging', 'level.help')
+        parser = self.cp.get('logging', 'level.parser')
+        self.cp.set('__main__', 'log_level.default', value)
+        self.cp.set('__main__', 'log_level.help', help)
+        self.cp.set('__main__', 'log_level.parser', parser)
+        self.cp.remove_option('logging', 'level.default')
+        self.cp.remove_option('logging', 'level.help')
+        self.cp.remove_option('logging', 'level.parser')
+        # parse it
+        self.cp.parse_all()
+        new_value = self.cp.get('logging', 'level')
+        self.assertEquals(new_value.value, new_value.parser(value))
+
+    def test_add_upgrade_hook(self):
+        """Test add_upgrade_hook method"""
+        self.cp.add_upgrade_hook('foo', 'bar', lambda x: None)
+        self.assertIn(('foo', 'bar'), self.cp.upgrade_hooks)
+        # try to add the same upgrade_hook
+        self.assertRaises(ValueError, self.cp.add_upgrade_hook, 'foo', 'bar',
+                          lambda x: None)
+
+    def test_ignore_one(self):
+        """Test ignore files config, one regex."""
+        conf_file = os.path.join(self.test_root, 'test_new_config.conf')
+        with open(conf_file, 'w') as fp:
+            fp.write('[__main__]\n')
+            fp.write('ignore = .*\\.pyc\n')  # all .pyc files
+        self.assertTrue(os.path.exists(conf_file))
+        self.cp.read([conf_file])
+        self.cp.parse_all()
+        self.assertEquals(self.cp.get('__main__', 'ignore').value,
+                          [r'.*\.pyc'])
+
+    def test_ignore_two(self):
+        """Test ignore files config, two regexes."""
+        conf_file = os.path.join(self.test_root, 'test_new_config.conf')
+        with open(conf_file, 'w') as fp:
+            fp.write('[__main__]\n')
+            fp.write('ignore = .*\\.pyc\n')  # all .pyc files
+            fp.write('         .*\\.sw[opnx]\n')  # all gvim temp files
+        self.assertTrue(os.path.exists(conf_file))
+        self.cp.read([conf_file])
+        self.cp.parse_all()
+        self.assertEquals(self.cp.get('__main__', 'ignore').value,
+                          ['.*\\.pyc', '.*\\.sw[opnx]'])
