@@ -14,7 +14,7 @@
 #
 # You should have received a copy of the GNU General Public License along
 # with this program.  If not, see <http://www.gnu.org/licenses/>.
-'''Tests the Hashs Queue.'''
+"""Tests the Hashs Queue."""
 
 from __future__ import with_statement
 
@@ -33,22 +33,22 @@ TESTS_DIR = os.path.join(os.getcwd(), "tmp")
 
 
 class EventListener(object):
-    '''Store the events.'''
+    """Store the events."""
     def __init__(self):
         self._events = []
 
     def handle_default(self, event, *args):
-        '''Store the received event.'''
+        """Store the received event."""
         self._events.append((event,) + args)
 
     def events(self):
-        '''Clean and return events.'''
+        """Clean and return events."""
         tmp = self._events
         self._events = []
         return tmp
 
 def hash_hack_run(self):
-    '''Function to hack the hash class to avoid it processing the requests.'''
+    """Function to hack the hash class to avoid it processing the requests."""
     while True:
         info = self.queue.get()
         path, mdid = info
@@ -56,11 +56,11 @@ def hash_hack_run(self):
 
 
 class DownloadFinishedTests(TwistedTestCase):
-    '''Test the AQ Download Finished Nanny behaviour.'''
+    """Test the AQ Download Finished Nanny behaviour."""
     timeout = 2
 
     def setUp(self):
-        '''set up the test.'''
+        """set up the test."""
         try:
             os.mkdir(TESTS_DIR)
         except OSError:
@@ -91,15 +91,15 @@ class DownloadFinishedTests(TwistedTestCase):
         fsm.set_node_id(self.tf, "nodeid")
 
     def tearDown(self):
-        '''tear down the test.'''
+        """tear down the test."""
         hash_queue._Hasher.run = self._original_hash_run
 
     def insert_in_hq(self, path, node_id):
-        '''Inserts something in HQ and waits that thread.'''
+        """Inserts something in HQ and waits that thread."""
         d = defer.Deferred()
         self.hq.insert(path, node_id)
         def wait():
-            '''waits for the var to get set'''
+            """waits for the var to get set"""
             if self.hq.hasher.hashing is None:
                 reactor.callLater(.1, wait)
             else:
@@ -108,11 +108,11 @@ class DownloadFinishedTests(TwistedTestCase):
         return d
 
     def release_hq(self):
-        '''Releases HQ as it finished.'''
+        """Releases HQ as it finished."""
         d = defer.Deferred()
         self.hq.insert(None, None)
         def wait():
-            '''waits for the var to get set'''
+            """waits for the var to get set"""
             if self.hq.hasher.hashing is not None:
                 reactor.callLater(.1, wait)
             else:
@@ -122,7 +122,7 @@ class DownloadFinishedTests(TwistedTestCase):
 
 
     def test_forward(self):
-        '''Forwards the event when file is not blocked.'''
+        """Forwards the event when file is not blocked."""
         self.eq.push("AQ_DOWNLOAD_COMMIT", "", "nodeid", "s_hash")
 
         def check(_):
@@ -138,7 +138,7 @@ class DownloadFinishedTests(TwistedTestCase):
         return d
 
     def test_blocks_when_open(self):
-        '''Blocks the event if the file is opened.'''
+        """Blocks the event if the file is opened."""
         self.eq.push("FS_FILE_OPEN", self.tf)
         self.eq.push("AQ_DOWNLOAD_COMMIT", "", "nodeid", "s_hash")
 
@@ -155,7 +155,7 @@ class DownloadFinishedTests(TwistedTestCase):
         return d
 
     def test_blocks_when_hashing(self):
-        '''Blocks the event if the file is being hashed.'''
+        """Blocks the event if the file is being hashed."""
         def check(_):
             self.assertEqual(self.listener.events(), [
                         ("AQ_DOWNLOAD_COMMIT", "", "nodeid", "s_hash"),
@@ -169,7 +169,7 @@ class DownloadFinishedTests(TwistedTestCase):
         return d
 
     def test_closenowrite(self):
-        '''A close_nowrite received, but no file was blocked.'''
+        """A close_nowrite received, but no file was blocked."""
         self.eq.push("FS_FILE_OPEN", self.tf)
         self.eq.push("AQ_DOWNLOAD_COMMIT", "", "nodeid", "s_hash")
 
@@ -194,7 +194,7 @@ class DownloadFinishedTests(TwistedTestCase):
         return d
 
     def test_blocks_closewrite(self):
-        '''Blocks the event and does NOT release it when close write.'''
+        """Blocks the event and does NOT release it when close write."""
         self.eq.push("FS_FILE_OPEN", self.tf)
         self.eq.push("AQ_DOWNLOAD_COMMIT", "", "nodeid", "s_hash")
 
@@ -221,7 +221,7 @@ class DownloadFinishedTests(TwistedTestCase):
         return d
 
     def test_blocks_release_close(self):
-        '''Blocks the event and releases it when close.'''
+        """Blocks the event and releases it when close."""
         self.eq.push("FS_FILE_OPEN", self.tf)
         self.eq.push("AQ_DOWNLOAD_COMMIT", "", "nodeid", "s_hash")
 
@@ -249,7 +249,7 @@ class DownloadFinishedTests(TwistedTestCase):
         return d
 
     def test_blocks_release_hash_doubleopen(self):
-        '''Blocks the event and releases it when hashed, double mixed open.'''
+        """Blocks the event and releases it when hashed, double mixed open."""
         self.eq.push("FS_FILE_OPEN", self.tf)
         self.eq.push("AQ_DOWNLOAD_COMMIT", "", "nodeid", "s_hash")
 
@@ -310,7 +310,7 @@ class DownloadFinishedTests(TwistedTestCase):
         return d
 
     def test_blocks_release_close_doubleopen(self):
-        '''Blocks the event and releases it when close, double open.'''
+        """Blocks the event and releases it when close, double open."""
         self.eq.push("FS_FILE_OPEN", self.tf)
         self.eq.push("AQ_DOWNLOAD_COMMIT", "", "nodeid", "s_hash")
         self.eq.push("FS_FILE_OPEN", self.tf)
@@ -349,7 +349,7 @@ class DownloadFinishedTests(TwistedTestCase):
         return d
 
     def test_blocks_release_close_differentfiles(self):
-        '''Blocks the event and releases it when close, several files.'''
+        """Blocks the event and releases it when close, several files."""
         self.eq.push("FS_FILE_OPEN", self.tf)
         self.eq.push("AQ_DOWNLOAD_COMMIT", "", "nodeid", "s_hash")
 
@@ -383,7 +383,7 @@ class DownloadFinishedTests(TwistedTestCase):
         return d
 
     def test_blocks_release_hashdone(self):
-        '''Blocks the event and releases it when the hash is done.'''
+        """Blocks the event and releases it when the hash is done."""
         def check1(_):
             self.assertEqual(self.listener.events(), [
                         ("AQ_DOWNLOAD_COMMIT", "", "nodeid", "s_hash"),
@@ -408,7 +408,7 @@ class DownloadFinishedTests(TwistedTestCase):
         return d
 
     def test_blocks_closewrite_hashdone(self):
-        '''Knows that is hashing also because of CLOSE_WRITE.'''
+        """Knows that is hashing also because of CLOSE_WRITE."""
         self.eq.push("FS_FILE_OPEN", self.tf)
         self.eq.push("AQ_DOWNLOAD_COMMIT", "", "nodeid", "s_hash")
 
@@ -448,7 +448,7 @@ class DownloadFinishedTests(TwistedTestCase):
         return d
 
     def test_create_discards(self):
-        '''The block and open count is discarded when file created.'''
+        """The block and open count is discarded when file created."""
         self.eq.push("FS_FILE_OPEN", self.tf)
         self.eq.push("AQ_DOWNLOAD_COMMIT", "", "nodeid", "s_hash")
 
@@ -475,7 +475,7 @@ class DownloadFinishedTests(TwistedTestCase):
         return d
 
     def test_create_noblocked(self):
-        '''Create is received, nothing was blocked.'''
+        """Create is received, nothing was blocked."""
         def check(_):
             self.assertEqual(self.listener.events(), [
                         ("FS_FILE_CREATE", self.tf),
@@ -488,7 +488,7 @@ class DownloadFinishedTests(TwistedTestCase):
         return d
 
     def test_delete_discards(self):
-        '''The block and open count is discarded when file deleted.'''
+        """The block and open count is discarded when file deleted."""
         self.eq.push("FS_FILE_OPEN", self.tf)
         self.eq.push("AQ_DOWNLOAD_COMMIT", "", "nodeid", "s_hash")
 
@@ -515,7 +515,7 @@ class DownloadFinishedTests(TwistedTestCase):
         return d
 
     def test_delete_noblocked(self):
-        '''Delete is received, nothing was blocked.'''
+        """Delete is received, nothing was blocked."""
         def check(_):
             self.assertEqual(self.listener.events(), [
                         ("FS_FILE_DELETE", self.tf),
@@ -528,7 +528,7 @@ class DownloadFinishedTests(TwistedTestCase):
         return d
 
     def test_file_move_changeblock(self):
-        '''Move event affects the blocked stuff.'''
+        """Move event affects the blocked stuff."""
         # prepare the second file
         tf2 = self.tf + "2"
         self.fsm.create(tf2, "")
@@ -569,7 +569,7 @@ class DownloadFinishedTests(TwistedTestCase):
         return d
 
     def test_dir_move_changeblock(self):
-        '''Move event affects the blocked stuff.'''
+        """Move event affects the blocked stuff."""
         # create several files with tricky names
         tf1 = os.path.join(self.usrdir, "fo123", "tfile")
         tf2 = os.path.join(self.usrdir, "fo12", "tfile")
@@ -640,7 +640,7 @@ class DownloadFinishedTests(TwistedTestCase):
         return d
 
     def test_complex(self):
-        '''Open, Open, Close, MoveFile, Open, MoveDir, Close, Close... test!'''
+        """Open, Open, Close, MoveFile, Open, MoveDir, Close, Close... test!"""
         # create a file inside a directory
         dir_from = os.path.join(self.usrdir, "foo")
         tf1 = os.path.join(self.usrdir, "foo", "tfile1")
@@ -696,7 +696,7 @@ class DownloadFinishedTests(TwistedTestCase):
             ])
             self.assertFalse(newtf2 in self.nanny._blocked)
             self.assertFalse(newtf2 in self.nanny._opened)
-        '''Open, Open, Close, MoveFile, Open, MoveDir, Close, Close... test!'''
+        """Open, Open, Close, MoveFile, Open, MoveDir, Close, Close... test!"""
 
         d = defer.Deferred()
         d.addCallback(check1)
@@ -714,7 +714,7 @@ class DownloadFinishedTests(TwistedTestCase):
         return d
 
     def test_mixed_hash_close(self):
-        '''It's ready to release according to hashing, but it's opened.'''
+        """It's ready to release according to hashing, but it's opened."""
         def check1(_):
             self.assertEqual(self.listener.events(), [
                         ("AQ_DOWNLOAD_COMMIT", "", "nodeid", "s_hash"),
@@ -755,7 +755,7 @@ class DownloadFinishedTests(TwistedTestCase):
         return d
 
     def test_mixed_close_hash(self):
-        '''It's ready to release according to open/close, but it's hashing.'''
+        """It's ready to release according to open/close, but it's hashing."""
         self.eq.push("FS_FILE_OPEN", self.tf)
         self.eq.push("AQ_DOWNLOAD_COMMIT", "", "nodeid", "s_hash")
 
