@@ -18,6 +18,7 @@
 """ Test file based persistent shelf """
 from __future__ import with_statement
 
+import cPickle
 import os
 import hashlib
 import unittest
@@ -46,7 +47,7 @@ class TestFileShelf(testcase.BaseTwistedTestCase):
         self.shelf = self.fileshelf_class(self.path)
 
     def tearDown(self):
-        '''Clean up the tests.'''
+        """Clean up the tests."""
         shutil.rmtree(self.path)
         testcase.BaseTwistedTestCase.tearDown(self)
 
@@ -132,7 +133,7 @@ class TestFileShelf(testcase.BaseTwistedTestCase):
         self.assertTrue(('foo', 'bar') and ('foo1', 'bar1') in \
                         [(k, v) for k, v in shelf.items()])
 
-    def test_broked_metadata_without_backup(self):
+    def test_broken_metadata_without_backup(self):
         """test the shelf behavior when it hit a broken metadata file without
         backup.
         """
@@ -147,7 +148,7 @@ class TestFileShelf(testcase.BaseTwistedTestCase):
             f.write(BROKEN_PICKLE)
         self.assertRaises(KeyError, self.shelf.__getitem__, 'broken_pickle')
 
-    def test_broked_metadata_with_backup(self):
+    def test_broken_metadata_with_backup(self):
         """test that each time a metadata file is updated a .old is kept"""
         self.shelf['bad_file'] = {'value':'old'}
         path = self.shelf.key_file('bad_file')
@@ -223,7 +224,71 @@ class TestFileShelf(testcase.BaseTwistedTestCase):
         self.assertFalse(os.path.exists(path+'.old'), 'there is a .old file!')
         self.assertFalse(os.path.exists(path+'.new'), 'there is a .new file!')
 
+    def test_custom_unpickle(self):
+        """Test the _pickle and _unpikle methods."""
+        self.mktemp('my_shelf')
+        class InMemoryFileShelf(FileShelf):
+            """A in-memory FileShelf."""
+            values = {}
 
+            def key_file(self, key):
+                """Noop key_file method."""
+                return key
+
+            def _check_and_create_dirs(self, path):
+                """Noop"""
+                pass
+
+            def _unpickle(self, fd):
+                """Custom _unpickle."""
+                return cPickle.loads(self.values[fd.name])
+
+            def _pickle(self, value, fd, protocol=2):
+                """Custom _pickle."""
+                key = fd.name.strip('.new')
+                self.values[key] = cPickle.dumps(value, protocol=protocol)
+
+        shelf = InMemoryFileShelf(self.path)
+        shelf['foo'] = 'bar'
+        self.assertIn('foo', shelf.values)
+        self.assertEquals(shelf.values['foo'],
+                          cPickle.dumps('bar', protocol=2))
+
+    def test_broken_metadata_iteritems(self):
+        """Test that broken metadata is ignored during iteritems."""
+        self.shelf['ok_key'] = {'status':'this is valid metadata'}
+        self.shelf['bad_file'] = {}
+        path = self.shelf.key_file('bad_file')
+        open(path, 'w').close()
+        self.assertRaises(KeyError, self.shelf.__getitem__, 'bad_file')
+        self.assertEquals(1, len(list(self.shelf.iteritems())))
+        self.assertFalse(os.path.exists(path))
+
+        self.shelf['broken_pickle'] = {}
+        path = self.shelf.key_file('broken_pickle')
+        with open(path, 'w') as f:
+            f.write(BROKEN_PICKLE)
+        self.assertRaises(KeyError, self.shelf.__getitem__, 'broken_pickle')
+        self.assertEquals(1, len(list(self.shelf.iteritems())))
+        self.assertFalse(os.path.exists(path))
+
+    def test_broken_metadata_items(self):
+        """Test that broken metadata is ignored during iteritems."""
+        self.shelf['ok_key'] = {'status':'this is valid metadata'}
+        self.shelf['bad_file'] = {}
+        path = self.shelf.key_file('bad_file')
+        open(path, 'w').close()
+        self.assertRaises(KeyError, self.shelf.__getitem__, 'bad_file')
+        self.assertEquals(1, len(list(self.shelf.items())))
+        self.assertFalse(os.path.exists(path))
+
+        self.shelf['broken_pickle'] = {}
+        path = self.shelf.key_file('broken_pickle')
+        with open(path, 'w') as f:
+            f.write(BROKEN_PICKLE)
+        self.assertRaises(KeyError, self.shelf.__getitem__, 'broken_pickle')
+        self.assertEquals(1, len(list(self.shelf.items())))
+        self.assertFalse(os.path.exists(path))
 
 
 class CachedFileShelfTests(TestFileShelf):
@@ -244,7 +309,7 @@ class CachedFileShelfTests(TestFileShelf):
         self.shelf['realkey']
         self.assertEquals(self.shelf.cache_hits, 1)
 
-    def test_broked_metadata_with_backup(self):
+    def test_broken_metadata_with_backup(self):
         """overrides parent test as we have the value in the cache."""
         self.shelf['bad_file'] = {'value':'old'}
         path = self.shelf.key_file('bad_file')
@@ -280,7 +345,7 @@ class CachedFileShelfTests(TestFileShelf):
 
 
 class LRUCacheTests(unittest.TestCase):
-    '''Test the LRUCache class'''
+    """Test the LRUCache class"""
     # I'm a unittest of a class, I need to access protected members
     # pylint: disable-msg=W0212
 
