@@ -27,11 +27,15 @@ import subprocess
 import random
 import dbus
 import os
-import socket, ssl, httplib, urllib
+import socket, httplib, urllib
 
 import gnomekeyring
 from oauth import oauth
-from ubuntuone.clientdefs import VERSION as ubuntuone_client_version
+try:
+    from ubuntuone.clientdefs import VERSION
+    ubuntuone_client_version = VERSION
+except ImportError:
+    ubuntuone_client_version = "Unknown"
 from ubuntuone.oauthdesktop.key_acls import set_all_key_acls
 
 from threading import Thread
@@ -53,19 +57,26 @@ NM_STATE_CONNECTED = 3
 NM_STATE_DISCONNECTED = 4
 
 # Monkeypatch httplib so that urllib will fail on invalid certificate
-def _connect_wrapper(self):
-    """Override HTTPSConnection.connect to require certificate checks"""
-    sock = socket.create_connection((self.host, self.port), self.timeout)
-    try:
-        if self._tunnel_host:
-            self.sock = sock
-            self._tunnel()
-    except AttributeError:
-        pass
-    self.sock = ssl.wrap_socket(sock, self.key_file, self.cert_file,
-        cert_reqs=ssl.CERT_REQUIRED, 
-        ca_certs="/etc/ssl/certs/ca-certificates.crt")
-httplib.HTTPSConnection.connect = _connect_wrapper
+# Only patch if we can import ssl to work around fail in 2.5
+try:
+    import ssl
+except ImportError:
+    pass
+else:
+    def _connect_wrapper(self):
+        """Override HTTPSConnection.connect to require certificate checks"""
+        sock = socket.create_connection((self.host, self.port), self.timeout)
+        try:
+            if self._tunnel_host:
+                self.sock = sock
+                self._tunnel()
+        except AttributeError:
+            pass
+        self.sock = ssl.wrap_socket(sock, self.key_file, self.cert_file,
+                cert_reqs=ssl.CERT_REQUIRED, 
+                ca_certs="/etc/ssl/certs/ca-certificates.crt")
+    httplib.HTTPSConnection.connect = _connect_wrapper
+
 
 class FancyURLOpenerWithRedirectedPOST(urllib.FancyURLopener):
     """FancyURLopener does not redirect postdata when redirecting POSTs"""
@@ -79,7 +90,7 @@ class FancyURLOpenerWithRedirectedPOST(urllib.FancyURLopener):
             newurl = headers['uri']
         else:
             return
-        void = fp.read()
+        fp.read()
         fp.close()
         # In case the server sent a relative URL, join with original:
         newurl = urllib.basejoin(self.type + ":" + url, newurl)
@@ -244,9 +255,10 @@ class AuthorisationClient(object):
             logger.debug("Token successfully requested")
             return out_token
         except:
+            error = Exception(data)
             logger.error("Token was not successfully retrieved: data was '%s'",
-               data)
-            self._forward_error_callback(oauth.OAuthError(data))
+               str(error))
+            self._forward_error_callback(error)
 
     def open_in_browser(self, url):
         """Open the given URL in the user's web browser."""
@@ -451,6 +463,8 @@ class TemporaryTwistedWebServer(resource.Resource):
         </html>
         """ % { 'url' : url }
         else:
+            self.retrieve_function(store=self.store_yes_no, verifier=verifier)
+            reactor.callLater(3, self.stop)
             request.setResponseCode(400)
             return """<!doctype html>
         <html><head><title>Error</title></head>
