@@ -17,18 +17,22 @@
 # with this program.  If not, see <http://www.gnu.org/licenses/>.
 """ DBUS interface module """
 
+import os
 import dbus.service
 import logging
+import uuid
 
 from dbus import DBusException
 from itertools import groupby, chain
+from xml.etree import ElementTree
 
 from twisted.internet import defer
 from twisted.python.failure import Failure
 
 from ubuntuone.syncdaemon.event_queue import EVENTS
 from ubuntuone.syncdaemon.interfaces import IMarker
-from ubuntuone.syncdaemon.config import get_user_config
+from ubuntuone.syncdaemon import config
+from ubuntuone.syncdaemon.volume_manager import Share, UDF
 
 
 # Disable the "Invalid Name" check here, as we have lots of DBus style names
@@ -41,6 +45,8 @@ DBUS_IFACE_EVENTS_NAME = DBUS_IFACE_NAME+'.Events'
 DBUS_IFACE_FS_NAME = DBUS_IFACE_NAME+'.FileSystem'
 DBUS_IFACE_SHARES_NAME = DBUS_IFACE_NAME+'.Shares'
 DBUS_IFACE_CONFIG_NAME = DBUS_IFACE_NAME+'.Config'
+DBUS_IFACE_FOLDERS_NAME = DBUS_IFACE_NAME+'.Folders'
+DBUS_IFACE_PUBLIC_FILES_NAME = DBUS_IFACE_NAME + '.PublicFiles'
 
 # NetworkManager State constants
 NM_STATE_UNKNOWN = 0
@@ -71,6 +77,7 @@ class DBusExposedObject(dbus.service.Object):
     """ Base class that provides some helper methods to
     DBus exposed objects.
     """
+    #__metaclass__ = InterfaceType
 
     def __init__(self, bus_name, path):
         """ creates the instance. """
@@ -90,6 +97,36 @@ class DBusExposedObject(dbus.service.Object):
         """ emit's a Error signal. """
         self.SignalError(signal, extra_args)
 
+    @classmethod
+    def _add_docstring(cls, func, reflection_data):
+        """add <docstring> tag to reflection_data if func.__doc__ isn't None."""
+        # add docstring element
+        if getattr(func, '__doc__', None) is not None:
+
+            element = ElementTree.fromstring(reflection_data)
+            doc = element.makeelement('docstring', dict())
+            data = '<![CDATA[' + func.__doc__ + ']]>'
+            doc.text = '%s'
+            element.insert(0, doc)
+            return ElementTree.tostring(element) % data
+        else:
+            return reflection_data
+
+    @classmethod
+    def _reflect_on_method(cls, func):
+        """override _reflect_on_method to provide an extra <docstring> element
+        in the xml.
+        """
+        reflection_data = dbus.service.Object._reflect_on_method(func)
+        reflection_data = cls._add_docstring(func, reflection_data)
+        return reflection_data
+
+    @classmethod
+    def _reflect_on_signal(cls, func):
+        reflection_data = dbus.service.Object._reflect_on_signal(func)
+        reflection_data = cls._add_docstring(func, reflection_data)
+        return reflection_data
+
 
 class Status(DBusExposedObject):
     """ Represent the status of the syncdaemon """
@@ -106,6 +143,22 @@ class Status(DBusExposedObject):
         DBusExposedObject.__init__(self, bus_name=bus_name,
                                    path=self.path)
 
+    def _get_current_state(self):
+        """Get the current status of the system."""
+        state = self.dbus_iface.main.state_manager.state
+        connection = self.dbus_iface.main.state_manager.connection.state
+        queues = self.dbus_iface.main.state_manager.queues.state.name
+        state_dict = {
+            'name': state.name,
+            'description': state.description,
+            'is_error': self.bool_str(state.is_error),
+            'is_connected': self.bool_str(state.is_connected),
+            'is_online': self.bool_str(state.is_online),
+            'queues': queues,
+            'connection': connection,
+        }
+        return state_dict
+
     @dbus.service.method(DBUS_IFACE_STATUS_NAME,
                          in_signature='', out_signature='a{ss}')
     def current_status(self):
@@ -113,14 +166,7 @@ class Status(DBusExposedObject):
         offline, trying_to_connect, server_rescan or online.
         """
         logger.debug('called current_status')
-        state = self.dbus_iface.main.state.state
-        self.emit_status_changed(state)
-        state_dict = {'name':state.name,
-                      'description':state.description,
-                      'is_error':self.bool_str(state.is_error),
-                      'is_connected':self.bool_str(state.is_connected),
-                      'is_online':self.bool_str(state.is_online)}
-        return state_dict
+        return self._get_current_state()
 
     @dbus.service.method(DBUS_IFACE_STATUS_NAME, out_signature='aa{ss}')
     def current_downloads(self):
@@ -148,6 +194,16 @@ class Status(DBusExposedObject):
             current_downloads.append(entry)
         return current_downloads
 
+    @dbus.service.method(DBUS_IFACE_STATUS_NAME, out_signature='as')
+    def waiting_metadata(self):
+        """Return a list of the operations in the meta-queue."""
+        logger.debug('called waiting_metadata')
+        waiting_metadata = []
+        for cmd in self.action_queue.meta_queue.full_queue():
+            if cmd is not None:
+                waiting_metadata.append(str(cmd))
+        return waiting_metadata
+
     @dbus.service.method(DBUS_IFACE_STATUS_NAME, out_signature='aa{ss}')
     def waiting_content(self):
         """
@@ -155,7 +211,9 @@ class Status(DBusExposedObject):
         """
         logger.debug('called waiting_content')
         waiting_content = []
-        for cmd in self.action_queue.content_queue.waiting:
+        for cmd in self.action_queue.content_queue.full_queue():
+            if cmd is None:
+                continue
             try:
                 if IMarker.providedBy(cmd.node_id):
                     # it's a marker! so it's the mdid :)
@@ -233,6 +291,11 @@ class Status(DBusExposedObject):
         """ Fire a D-BUS signal, notifying an upload has finished.  """
         pass
 
+    @dbus.service.signal(DBUS_IFACE_STATUS_NAME, signature='say')
+    def InvalidName(self, dirname, filename):
+        """Fire a D-BUS signal, notifying an invalid file or dir name."""
+        pass
+
     @dbus.service.signal(DBUS_IFACE_STATUS_NAME)
     def StatusChanged(self, status):
         """ Fire a D-BUS signal, notifying that the status of the
@@ -255,9 +318,7 @@ class Status(DBusExposedObject):
         """
 
     def emit_content_queue_changed(self, head, waiting):
-        """
-        Emits the signal.
-        """
+        """Emits the signal."""
         if head is None or getattr(head, 'node_id', None) is None:
             head_path = ''
             head_cmd = ''
@@ -283,14 +344,13 @@ class Status(DBusExposedObject):
                            'size': '0'}
         self.ContentQueueChanged(message)
 
+    def emit_invalid_name(self, dirname, filename):
+        """Emits the signal."""
+        self.InvalidName(unicode(dirname), str(filename))
+
     def emit_status_changed(self, state):
-        """ Emits the signal """
-        state_dict = {'name':state.name,
-                      'description':self.bool_str(state.description),
-                      'is_error':self.bool_str(state.is_error),
-                      'is_connected':self.bool_str(state.is_connected),
-                      'is_online':self.bool_str(state.is_online)}
-        self.StatusChanged(state_dict)
+        """Emits the signal."""
+        self.StatusChanged(self._get_current_state())
 
     def emit_download_started(self, download):
         """ Emits the signal """
@@ -501,16 +561,16 @@ class EventListener(object):
                         upload_error=str(error))
             self.dbus_iface.status.emit_signal_error('UploadFinished', args)
 
+    def handle_FS_INVALID_NAME(self, dirname, filename):
+        """Handle FS_INVALID_NAME."""
+        self.handle_default('FS_INVALID_NAME', dirname, filename)
+        self.dbus_iface.status.emit_invalid_name(dirname, filename)
+
     def handle_SYS_STATE_CHANGED(self, state):
         """ handle SYS_STATE_CHANGED """
         self.handle_default('SYS_STATE_CHANGED', state)
         self.dbus_iface.status.emit_status_changed(state)
 
-    def handle_SV_SHARE_CHANGED(self, message, share):
-        """ handle SV_SHARE_CHANGED event, emit's ShareChanged signal. """
-        self.handle_default('SV_SHARE_CHANGED', message, share)
-        self.dbus_iface.shares.emit_share_changed(message, share)
-
     def handle_SV_FREE_SPACE(self, share_id, free_bytes):
         """ handle SV_FREE_SPACE event, emit ShareChanged signal. """
         self.handle_default('SV_FREE_SPACE', share_id, free_bytes)
@@ -519,13 +579,13 @@ class EventListener(object):
     def handle_AQ_CREATE_SHARE_OK(self, share_id, marker):
         """ handle AQ_CREATE_SHARE_OK event, emit's ShareCreated signal. """
         self.handle_default('AQ_CREATE_SHARE_OK', share_id, marker)
-        share = self.dbus_iface.volume_manager.shared.get(share_id)
+        share = self.dbus_iface.volume_manager.shared.get(str(share_id))
         share_dict = {}
         if share:
             # pylint: disable-msg=W0212
             share_dict.update(self.dbus_iface.shares._get_share_dict(share))
         else:
-            share_dict.update(dict(share_id=share_id))
+            share_dict.update(dict(volume_id=str(share_id)))
         self.dbus_iface.shares.emit_share_created(share_dict)
 
     def handle_AQ_CREATE_SHARE_ERROR(self, marker, error):
@@ -537,15 +597,125 @@ class EventListener(object):
 
     def handle_AQ_ANSWER_SHARE_OK(self, share_id, answer):
         """ handle AQ_ANSWER_SHARE_OK event, emit's ShareAnswerOk signal. """
-        self.handle_default('AQ_ANSWER_SHARE_OK', share_id, answer)
-        self.dbus_iface.shares.emit_share_answer_response(share_id, answer)
+        self.handle_default('AQ_ANSWER_SHARE_OK', str(share_id), answer)
+        self.dbus_iface.shares.emit_share_answer_response(str(share_id), answer)
 
     def handle_AQ_ANSWER_SHARE_ERROR(self, share_id, answer, error):
         """ handle AQ_ANSWER_SHARE_ERROR event, emit's ShareAnswerError signal.
         """
-        self.handle_default('AQ_ANSWER_SHARE_ERROR', share_id, answer, error)
-        self.dbus_iface.shares.emit_share_answer_response(share_id, answer,
+        self.handle_default('AQ_ANSWER_SHARE_ERROR', str(share_id), answer, error)
+        self.dbus_iface.shares.emit_share_answer_response(str(share_id), answer,
                                                           error)
+    def handle_VM_UDF_SUBSCRIBED(self, udf):
+        """Handle VM_UDF_SUBSCRIBED event, emit FolderSubscribed signal."""
+        self.handle_default('VM_UDF_SUBSCRIBED', udf)
+        self.dbus_iface.folders.emit_folder_subscribed(udf)
+
+    def handle_VM_UDF_SUBSCRIBE_ERROR(self, udf_id, error):
+        """
+        Handle VM_UDF_SUBSCRIBE_ERROR event, emit
+        FolderSubscribeError signal.
+
+        """
+        self.handle_default('VM_UDF_SUBSCRIBE_ERROR', udf_id, error)
+        self.dbus_iface.folders.emit_folder_subscribe_error(udf_id, error)
+
+    def handle_VM_UDF_UNSUBSCRIBED(self, udf):
+        """Handle VM_UDF_UNSUBSCRIBED event, emit FolderUnSubscribed signal."""
+        self.handle_default('VM_UDF_UNSUBSCRIBED', udf)
+        self.dbus_iface.folders.emit_folder_unsubscribed(udf)
+
+    def handle_VM_UDF_UNSUBSCRIBE_ERROR(self, udf_id, error):
+        """Handle VM_UDF_UNSUBSCRIBE_ERROR event.
+
+        Emit FolderUnSubscribeError signal.
+
+        """
+        self.handle_default('VM_UDF_UNSUBSCRIBE_ERROR', udf_id, error)
+        self.dbus_iface.folders.emit_folder_unsubscribe_error(udf_id, error)
+
+    def handle_VM_UDF_CREATED(self, udf):
+        """Handle VM_UDF_CREATED event, emit FolderCreated signal."""
+        self.handle_default('VM_UDF_CREATED', udf)
+        self.dbus_iface.folders.emit_folder_created(udf)
+
+    def handle_VM_UDF_CREATE_ERROR(self, path, error):
+        """Handle VM_UDF_CREATE_ERROR event, emit FolderCreateError signal."""
+        self.handle_default('VM_UDF_CREATE_ERROR', path, error)
+        self.dbus_iface.folders.emit_folder_create_error(path, error)
+
+    def handle_VM_SHARE_CREATED(self, share_id):
+        """Handle VM_SHARE_CREATED event, emit NewShare event."""
+        self.handle_default('VM_SHARE_CREATED', share_id)
+        self.dbus_iface.shares.emit_new_share(share_id)
+
+    def handle_VM_VOLUME_DELETED(self, volume):
+        """Handle VM_VOLUME_DELETED event.
+
+        Emits FolderDeleted or ShareChanged signal.
+
+        """
+        self.handle_default('VM_VOLUME_DELETED', getattr(volume, 'id', volume))
+        if isinstance(volume, Share):
+            self.dbus_iface.shares.emit_share_changed('deleted', volume)
+        elif isinstance(volume, UDF):
+            self.dbus_iface.folders.emit_folder_deleted(volume)
+        else:
+            logger.error("Unable to handle VM_VOLUME_DELETE for "
+                     "volume_id=%r as it's not a share or UDF", volume.id)
+
+    def handle_VM_VOLUME_DELETE_ERROR(self, volume_id, error):
+        """Handle VM_SHARE_DELETED event, emit ShareDeleted event."""
+        self.handle_default('VM_VOLUME_DELETE_ERROR', volume_id, error)
+        try:
+            volume = self.dbus_iface.volume_manager.get_volume(volume_id)
+        except KeyError:
+            logger.error("Unable to handle VM_VOLUME_DELETE_ERROR for "
+                         "volume_id=%r", volume_id)
+        else:
+            if isinstance(volume, Share):
+                self.dbus_iface.shares.emit_share_delete_error(volume, error)
+            elif isinstance(volume, UDF):
+                self.dbus_iface.folders.emit_folder_delete_error(volume, error)
+            else:
+                logger.error("Unable to handle VM_VOLUME_DELETE_ERROR for "
+                         "volume_id=%r as it's not a share or UDF", volume_id)
+
+    def handle_VM_SHARE_CHANGED(self, share_id):
+        """ handle VM_SHARE_CHANGED event, emit's ShareChanged signal. """
+        self.handle_default('VM_SHARE_CHANGED', share_id)
+        share = self.dbus_iface.volume_manager.shares.get(share_id)
+        self.dbus_iface.shares.emit_share_changed('changed', share)
+
+    def handle_AQ_CHANGE_PUBLIC_ACCESS_OK(self, share_id, node_id,
+                                          is_public, public_url):
+        """Handle the AQ_CHANGE_PUBLIC_ACCESS_OK event."""
+        self.handle_default('AQ_CHANGE_PUBLIC_ACCESS_OK', share_id, node_id,
+                            is_public, public_url)
+        self.dbus_iface.public_files.emit_public_access_changed(
+            share_id, node_id, is_public, public_url)
+
+    def handle_AQ_CHANGE_PUBLIC_ACCESS_ERROR(self, share_id, node_id, error):
+        """Handle the AQ_CHANGE_PUBLIC_ACCESS_ERROR event."""
+        self.handle_default('AQ_CHANGE_PUBLIC_ACCESS_ERROR',
+                            share_id, node_id, error)
+        self.dbus_iface.public_files.emit_public_access_change_error(
+            share_id, node_id, error)
+
+    def handle_SYS_ROOT_MISMATCH(self, root_id, new_root_id):
+        """Handle the SYS_ROOT_MISMATCH event."""
+        self.handle_default('SYS_ROOT_MISMATCH', root_id, new_root_id)
+        self.dbus_iface.sync.emit_root_mismatch(root_id, new_root_id)
+
+    def handle_AQ_PUBLIC_FILES_LIST_OK(self, public_files):
+        """Handle the AQ_PUBLIC_FILES_LIST_OK event."""
+        self.handle_default('AQ_PUBLIC_FILES_LIST_OK', public_files)
+        self.dbus_iface.public_files.emit_public_files_list(public_files)
+
+    def handle_AQ_PUBLIC_FILES_LIST_ERROR(self, error):
+        """Handle the AQ_PUBLIC_FILES_LIST_ERROR event."""
+        self.handle_default('AQ_PUBLIC_FILES_LIST_ERROR', error)
+        self.dbus_iface.public_files.emit_public_files_list_error(error)
 
 
 class SyncDaemon(DBusExposedObject):
@@ -565,14 +735,14 @@ class SyncDaemon(DBusExposedObject):
                          in_signature='', out_signature='')
     def connect(self):
         """ Connect to the server. """
-        logger.debug('connect requetsed')
+        logger.debug('connect requested')
         self.dbus_iface.connect()
 
     @dbus.service.method(DBUS_IFACE_SYNC_NAME,
                          in_signature='', out_signature='')
     def disconnect(self):
         """ Disconnect from the server. """
-        logger.debug('disconnect requetsed')
+        logger.debug('disconnect requested')
         self.dbus_iface.disconnect()
 
     @dbus.service.method(DBUS_IFACE_SYNC_NAME,
@@ -614,6 +784,16 @@ class SyncDaemon(DBusExposedObject):
             reply_handler()
         self.dbus_iface.quit()
 
+    @dbus.service.signal(DBUS_IFACE_SYNC_NAME,
+                         signature='ss')
+    def RootMismatch(self, root_id, new_root_id):
+        """RootMismatch signal, the user connected with a different account."""
+        pass
+
+    def emit_root_mismatch(self, root_id, new_root_id):
+        """Emit RootMismatch signal."""
+        self.RootMismatch(root_id, new_root_id)
+
 
 class FileSystem(DBusExposedObject):
     """ A dbus interface to the FileSystem Manager. """
@@ -641,6 +821,7 @@ class FileSystem(DBusExposedObject):
         if mdobj.__dict__.get('info', None):
             for k, v in mdobj.info.__dict__.items():
                 md_dict['info_'+str(k)] = str(v)
+        md_dict['path'] = path
         return md_dict
 
 
@@ -696,6 +877,20 @@ class Shares(DBusExposedObject):
             error_handler(ValueError("The share with id: %s don't exists" % \
                                      str(share_id)))
 
+    @dbus.service.method(DBUS_IFACE_SHARES_NAME,
+                         in_signature='s', out_signature='',
+                         async_callbacks=('reply_handler', 'error_handler'))
+    def delete_share(self, share_id, reply_handler=None, error_handler=None):
+        """Delete a Share."""
+        logger.debug('delete_share: %r', share_id)
+        try:
+            self.vm.delete_volume(str(share_id))
+            reply_handler()
+        except KeyError:
+            error_handler(ValueError("The volume with id: %s don't exists" % \
+                                     str(share_id)))
+
+
     @dbus.service.signal(DBUS_IFACE_SHARES_NAME,
                          signature='a{ss}')
     def ShareChanged(self, share_dict):
@@ -709,6 +904,13 @@ class Shares(DBusExposedObject):
         share attributes. """
         pass
 
+    @dbus.service.signal(DBUS_IFACE_SHARES_NAME,
+                         signature='a{ss}s')
+    def ShareDeleteError(self, share_dict, error):
+        """ A share was deleted, share_dict contains all available
+        share attributes. """
+        pass
+
     def emit_share_changed(self, message, share):
         """ emits ShareChanged or ShareDeleted signal for the share
         notification.
@@ -716,9 +918,12 @@ class Shares(DBusExposedObject):
         if message == 'deleted':
             self.ShareDeleted(self._get_share_dict(share))
         elif message == 'changed':
-            share = self.vm.shares[share.share_id]
             self.ShareChanged(self._get_share_dict(share))
 
+    def emit_share_delete_error(self, share, error):
+        """Emits ShareDeleteError signal."""
+        self.ShareDeleteError(self._get_share_dict(share), error)
+
     def emit_free_space(self, share_id, free_bytes):
         """ emits ShareChanged when free space changes """
         if share_id in self.vm.shares:
@@ -735,6 +940,8 @@ class Shares(DBusExposedObject):
                 share_dict[unicode(k)] = ''
             elif k == 'path':
                 share_dict[unicode(k)] = v.decode('utf-8')
+            elif k == 'accepted':
+                share_dict[unicode(k)] = self.bool_str(v)
             else:
                 share_dict[unicode(k)] = unicode(v)
         return share_dict
@@ -761,6 +968,21 @@ class Shares(DBusExposedObject):
             raise ValueError("path '%r' does not exist" % path)
         self.vm.create_share(path, username, name, access_level)
 
+    @dbus.service.method(DBUS_IFACE_SHARES_NAME,
+                         in_signature='sasss', out_signature='')
+    def create_shares(self, path, usernames, name, access_level):
+        """Share a subtree with several users at once.
+
+        @param path: that path to share (the root of the subtree)
+        @param usernames: the user names to offer the share to
+        @param name: the name of the share
+        @param access_level: 'View' or 'Modify'
+        """
+        logger.debug('create shares: %r, %r, %r, %r',
+                     path, usernames, name, access_level)
+        for user in usernames:
+            self.create_share(path, user, name, access_level)
+
     @dbus.service.signal(DBUS_IFACE_SHARES_NAME,
                          signature='a{ss}')
     def ShareCreated(self, share_info):
@@ -778,7 +1000,7 @@ class Shares(DBusExposedObject):
         self.ShareCreated(share_info)
 
     def emit_share_create_error(self, share_info, error):
-        """ emits ShareDeleted signal """
+        """Emit ShareCreateError signal."""
         path = self.fs_manager.get_by_mdid(str(share_info['marker'])).path
         share_info.update(dict(path=path))
         self.ShareCreateError(share_info, error)
@@ -812,12 +1034,23 @@ class Shares(DBusExposedObject):
         pass
 
     def emit_share_answer_response(self, share_id, answer, error=None):
-        """ emits ShareCreated signal """
-        answer_info = dict(share_id=share_id, answer=answer)
+        """Emits ShareAnswerResponse signal."""
+        answer_info = dict(volume_id=share_id, answer=answer)
         if error:
             answer_info['error'] = error
         self.ShareAnswerResponse(answer_info)
 
+    @dbus.service.signal(DBUS_IFACE_SHARES_NAME,
+                         signature='a{ss}')
+    def NewShare(self, share_info):
+        """A new share notification."""
+        pass
+
+    def emit_new_share(self, share_id):
+        """Emits NewShare signal."""
+        share = self.vm.get_volume(share_id)
+        self.NewShare(self._get_share_dict(share))
+
 
 class Config(DBusExposedObject):
     """ The Syncdaemon config/settings dbus interface. """
@@ -841,6 +1074,7 @@ class Config(DBusExposedObject):
         configured.
         The values are bytes/second
         """
+        logger.debug("called get_throttling_limits")
         try:
             aq = self.dbus_iface.action_queue
             download = -1
@@ -868,12 +1102,13 @@ class Config(DBusExposedObject):
     def set_throttling_limits(self, download, upload,
                          reply_handler=None, error_handler=None):
         """Set the read and write limits. The expected values are bytes/sec."""
+        logger.debug("called set_throttling_limits")
         try:
             # modify and save the config file
-            config = get_user_config()
-            config.set_throttling_read_limit(download)
-            config.set_throttling_write_limit(upload)
-            config.save()
+            user_config = config.get_user_config()
+            user_config.set_throttling_read_limit(download)
+            user_config.set_throttling_write_limit(upload)
+            user_config.save()
             # modify AQ settings
             aq = self.dbus_iface.action_queue
             if download == -1:
@@ -928,11 +1163,14 @@ class Config(DBusExposedObject):
     def _set_throttling_enabled(self, enabled):
         """set throttling enabled value and save the config"""
         # modify and save the config file
-        config = get_user_config()
-        config.set_throttling(enabled)
-        config.save()
+        user_config = config.get_user_config()
+        user_config.set_throttling(enabled)
+        user_config.save()
         # modify AQ settings
-        self.dbus_iface.action_queue.enable_throttling(enabled)
+        if enabled:
+            self.dbus_iface.action_queue.enable_throttling()
+        else:
+            self.dbus_iface.action_queue.disable_throttling()
 
     @dbus.service.method(DBUS_IFACE_CONFIG_NAME,
                          in_signature='', out_signature='b',
@@ -942,12 +1180,335 @@ class Config(DBusExposedObject):
         """Returns True (actually 1) if bandwidth throttling is enabled and
         False (0) otherwise.
         """
-        enabled = self.dbus_iface.action_queue.throttling
+        enabled = self.dbus_iface.action_queue.throttling_enabled
         if reply_handler:
             reply_handler(enabled)
         else:
             return enabled
 
+    @dbus.service.method(DBUS_IFACE_CONFIG_NAME,
+                         in_signature='', out_signature='b')
+    def udf_autosubscribe_enabled(self):
+        """Return the udf_autosubscribe config value."""
+        return config.get_user_config().get_udf_autosubscribe()
+
+    @dbus.service.method(DBUS_IFACE_CONFIG_NAME,
+                         in_signature='', out_signature='')
+    def enable_udf_autosubscribe(self):
+        """Enable UDF autosubscribe."""
+        user_config = config.get_user_config()
+        user_config.set_udf_autosubscribe(True)
+        user_config.save()
+
+    @dbus.service.method(DBUS_IFACE_CONFIG_NAME,
+                         in_signature='', out_signature='')
+    def disable_udf_autosubscribe(self):
+        """Enable UDF autosubscribe."""
+        user_config = config.get_user_config()
+        user_config.set_udf_autosubscribe(False)
+        user_config.save()
+
+    @dbus.service.method(DBUS_IFACE_CONFIG_NAME,
+                         in_signature='b', out_signature='')
+    def set_files_sync_enabled(self, enabled):
+        """Enable UDF autosubscribe."""
+        logger.debug('called set_files_sync_enabled %d', enabled)
+        user_config = config.get_user_config()
+        user_config.set_files_sync_enabled(enabled)
+        user_config.save()
+
+    @dbus.service.method(DBUS_IFACE_CONFIG_NAME,
+                         in_signature='', out_signature='b')
+    def files_sync_enabled(self):
+        logger.debug('called files_sync_enabled')
+        """Return the udf_autosubscribe config value."""
+        return config.get_user_config().get_files_sync_enabled()
+
+
+class Folders(DBusExposedObject):
+    """A dbus interface to interact with User Defined Folders"""
+
+    def __init__(self, bus_name, volume_manager, fs_manager):
+        """ Creates the instance. """
+        self.vm = volume_manager
+        self.fs = fs_manager
+        self.path = '/folders'
+        DBusExposedObject.__init__(self, bus_name=bus_name,
+                                   path=self.path)
+
+    def _get_udf_dict(self, udf):
+        """ get a dict with all the attributes of: udf. """
+        udf_dict = udf.__dict__.copy()
+        for k, v in udf_dict.items():
+            if v is None:
+                udf_dict[unicode(k)] = ''
+            elif k == 'subscribed':
+                udf_dict[unicode(k)] = self.bool_str(v)
+            elif k == 'path':
+                udf_dict[unicode(k)] = v.decode('utf-8')
+            else:
+                udf_dict[unicode(k)] = unicode(v)
+        return udf_dict
+
+    @dbus.service.method(DBUS_IFACE_FOLDERS_NAME, in_signature='s')
+    def create(self, path):
+        """Create a user defined folder in the specified path."""
+        logger.debug('Folders.create: %r', path)
+        try:
+            path = os.path.normpath(path)
+            self.vm.create_udf(path.encode('utf-8'))
+        except Exception, e:
+            logger.exception('Error while creating udf: %r', path)
+            self.emit_folder_create_error(path, str(e))
+
+    @dbus.service.method(DBUS_IFACE_FOLDERS_NAME, in_signature='s')
+    def delete(self, folder_id):
+        """Delete the folder specified by folder_id"""
+        logger.debug('Folders.delete: %r', folder_id)
+        try:
+            self.vm.delete_volume(folder_id)
+        except Exception, e:
+            logger.exception('Error while deleting volume: %r', folder_id)
+            self.emit_folder_delete_error({'id':folder_id}, str(e))
+
+    @dbus.service.method(DBUS_IFACE_FOLDERS_NAME, out_signature='aa{ss}')
+    def get_folders(self):
+        """Return the list of folders (a list of dicts)"""
+        logger.debug('Folders.get_folders')
+        return [self._get_udf_dict(udf) for udf in self.vm.udfs.values()]
+
+    @dbus.service.method(DBUS_IFACE_FOLDERS_NAME,
+                         in_signature='s', out_signature='')
+    def subscribe(self, folder_id):
+        """Subscribe to the specified folder"""
+        logger.debug('Folders.subscribe: %r', folder_id)
+        try:
+            self.vm.subscribe_udf(folder_id)
+        except Exception:
+            logger.exception('Error while subscribing udf: %r', folder_id)
+            raise
+
+    @dbus.service.method(DBUS_IFACE_FOLDERS_NAME,
+                         in_signature='s', out_signature='')
+    def unsubscribe(self, folder_id):
+        """Unsubscribe from the specified folder"""
+        logger.debug('Folders.unsubscribe: %r', folder_id)
+        try:
+            self.vm.unsubscribe_udf(folder_id)
+        except Exception:
+            logger.exception('Error while unsubscribing udf: %r', folder_id)
+            raise
+
+    @dbus.service.method(DBUS_IFACE_FOLDERS_NAME,
+                         in_signature='s', out_signature='a{ss}')
+    def get_info(self, path):
+        """Returns a dict containing the folder information."""
+        logger.debug('Folders.get_info: %r', path)
+        mdobj = self.fs.get_by_path(path.encode('utf-8'))
+        udf = self.vm.udfs.get(mdobj.share_id, None)
+        if udf is None:
+            return dict()
+        else:
+            return self._get_udf_dict(udf)
+
+    @dbus.service.method(DBUS_IFACE_FOLDERS_NAME,
+                         in_signature='', out_signature='')
+    def refresh_volumes(self):
+        """Refresh the volumes list, requesting it to the server."""
+        self.vm.refresh_volumes()
+
+    def emit_folder_created(self, folder):
+        """Emit the FolderCreated signal"""
+        udf_dict = self._get_udf_dict(folder)
+        self.FolderCreated(udf_dict)
+
+    @dbus.service.signal(DBUS_IFACE_FOLDERS_NAME,
+                         signature='a{ss}')
+    def FolderCreated(self, folder_info):
+        """Notify the creation of a user defined folder."""
+        pass
+
+    def emit_folder_create_error(self, path, error):
+        """Emit the FolderCreateError signal"""
+        info = dict(path=path.decode('utf-8'))
+        self.FolderCreateError(info, str(error))
+
+    @dbus.service.signal(DBUS_IFACE_FOLDERS_NAME,
+                         signature='a{ss}s')
+    def FolderCreateError(self, folder_info, error):
+        """Notify an error during the creation of a user defined folder."""
+        pass
+
+    def emit_folder_deleted(self, folder):
+        """Emit the FolderCreated signal"""
+        udf_dict = self._get_udf_dict(folder)
+        self.FolderDeleted(udf_dict)
+
+    @dbus.service.signal(DBUS_IFACE_FOLDERS_NAME,
+                         signature='a{ss}')
+    def FolderDeleted(self, folder_info):
+        """Notify the deletion of a user defined folder."""
+        pass
+
+    def emit_folder_delete_error(self, folder, error):
+        """Emit the FolderCreateError signal"""
+        udf_dict = self._get_udf_dict(folder)
+        self.FolderDeleteError(udf_dict, str(error))
+
+    @dbus.service.signal(DBUS_IFACE_FOLDERS_NAME,
+                         signature='a{ss}s')
+    def FolderDeleteError(self, folder_info, error):
+        """Notify an error during the deletion of a user defined folder."""
+        pass
+
+    def emit_folder_subscribed(self, folder):
+        """Emit the FolderSubscribed signal"""
+        udf_dict = self._get_udf_dict(folder)
+        self.FolderSubscribed(udf_dict)
+
+    @dbus.service.signal(DBUS_IFACE_FOLDERS_NAME,
+                         signature='a{ss}')
+    def FolderSubscribed(self, folder_info):
+        """Notify the subscription to a user defined folder."""
+        pass
+
+    def emit_folder_subscribe_error(self, folder_id, error):
+        """Emit the FolderSubscribeError signal"""
+        self.FolderSubscribeError({'id':folder_id}, str(error))
+
+    @dbus.service.signal(DBUS_IFACE_FOLDERS_NAME,
+                         signature='a{ss}s')
+    def FolderSubscribeError(self, folder_info, error):
+        """Notify an error while subscribing to a user defined folder."""
+        pass
+
+    def emit_folder_unsubscribed(self, folder):
+        """Emit the FolderUnSubscribed signal"""
+        udf_dict = self._get_udf_dict(folder)
+        self.FolderUnSubscribed(udf_dict)
+
+    @dbus.service.signal(DBUS_IFACE_FOLDERS_NAME,
+                         signature='a{ss}')
+    def FolderUnSubscribed(self, folder_info):
+        """Notify the unsubscription to a user defined folder."""
+        pass
+
+    def emit_folder_unsubscribe_error(self, folder_id, error):
+        """Emit the FolderUnSubscribeError signal"""
+        self.FolderUnSubscribeError({'id':folder_id}, str(error))
+
+    @dbus.service.signal(DBUS_IFACE_FOLDERS_NAME,
+                         signature='a{ss}s')
+    def FolderUnSubscribeError(self, folder_info, error):
+        """Notify an error while unsubscribing from a user defined folder."""
+        pass
+
+
+class PublicFiles(DBusExposedObject):
+    """A DBus interface for handling public files."""
+
+    def __init__(self, bus_name, fs_manager, action_queue):
+        self.fs = fs_manager
+        self.aq = action_queue
+        self.path = '/publicfiles'
+        DBusExposedObject.__init__(self, bus_name=bus_name, path=self.path)
+
+    @dbus.service.method(DBUS_IFACE_PUBLIC_FILES_NAME,
+                         in_signature='ssb', out_signature='')
+    def change_public_access(self, share_id, node_id, is_public):
+        """Change the public access of a file."""
+        logger.debug('PublicFiles.change_public_access: %r, %r, %r',
+                     share_id, node_id, is_public)
+        if share_id:
+            share_id = uuid.UUID(share_id)
+        else:
+            share_id = None
+        node_id = uuid.UUID(node_id)
+        self.aq.change_public_access(share_id, node_id, is_public)
+
+    @dbus.service.method(DBUS_IFACE_PUBLIC_FILES_NAME)
+    def get_public_files(self):
+        """Request the list of public files to the server.
+
+        The result will be send in a PublicFilesList signal.
+        """
+        self.aq.get_public_files()
+
+    def emit_public_access_changed(self, share_id, node_id, is_public,
+                                   public_url):
+        """Emit the PublicAccessChanged signal."""
+        share_id = str(share_id) if share_id else ''
+        node_id = str(node_id)
+        try:
+            relpath = self.fs.get_by_node_id(share_id,
+                                             node_id).path
+        except KeyError:
+            pass
+        else:
+            path=self.fs.get_abspath(share_id, relpath)
+        self.PublicAccessChanged(dict(
+                share_id=str(share_id) if share_id else '',
+                node_id=str(node_id),
+                is_public=self.bool_str(is_public),
+                public_url=public_url if public_url else '',
+                path=path))
+
+    @dbus.service.signal(DBUS_IFACE_PUBLIC_FILES_NAME,
+                         signature='a{ss}')
+    def PublicAccessChanged(self, file_info):
+        """Notify the new public access state of a file."""
+
+    def emit_public_access_change_error(self, share_id, node_id, error):
+        """Emit the PublicAccessChangeError signal."""
+        try:
+            relpath = self.fs.get_by_node_id(share_id,
+                                             node_id).path
+        except KeyError:
+            pass
+        else:
+            path=self.fs.get_abspath(share_id, relpath)
+        self.PublicAccessChangeError(dict(
+                share_id=str(share_id) if share_id else '',
+                node_id=str(node_id),
+                path=path), str(error))
+
+    @dbus.service.signal(DBUS_IFACE_PUBLIC_FILES_NAME,
+                         signature='a{ss}s')
+    def PublicAccessChangeError(self, file_info, error):
+        """Report an error in changing the public access of a file."""
+
+    @dbus.service.signal(DBUS_IFACE_PUBLIC_FILES_NAME,
+                        signature='aa{ss}')
+    def PublicFilesList(self, files):
+        """Notify the list of public files."""
+
+    @dbus.service.signal(DBUS_IFACE_PUBLIC_FILES_NAME,
+                         signature='s')
+    def PublicFilesListError(self, error):
+        """Report an error in geting the public files list."""
+
+    def emit_public_files_list(self, public_files):
+        """Emit the PublicFilesList signal."""
+        files = []
+        for pf in public_files:
+            try:
+                volume_id = str(pf['volume_id'])
+                node_id = str(pf['node_id'])
+                public_url = str(pf['public_url'])
+                relpath = self.fs.get_by_node_id(volume_id,
+                                                 pf['node_id']).path
+            except KeyError:
+                pass
+            else:
+                path = self.fs.get_abspath(volume_id, relpath).decode('utf-8')
+                files.append(dict(volume_id=volume_id, node_id=node_id,
+                                  public_url=public_url, path=path))
+        self.PublicFilesList(files)
+
+    def emit_public_files_list_error(self, error):
+        """Emit the PublicFilesListError signal."""
+        self.PublicFilesListError(error)
+
 
 class DBusInterface(object):
     """ Holder of all DBus exposed objects """
@@ -972,6 +1533,10 @@ class DBusInterface(object):
         self.fs = FileSystem(self.busName, self.fs_manager)
         self.shares = Shares(self.busName, self.fs_manager,
                              self.volume_manager)
+        self.folders = Folders(self.busName, self.volume_manager,
+                               self.fs_manager)
+        self.public_files = PublicFiles(
+            self.busName, self.fs_manager, self.action_queue)
         self.config = Config(self.busName, self)
         if system_bus is None and not DBusInterface.test:
             logger.debug('using the real system bus')
@@ -1040,6 +1605,7 @@ class DBusInterface(object):
         self.shares.remove_from_connection()
         self.config.remove_from_connection()
         self.event_queue.unsubscribe(self.event_listener)
+        self.folders.remove_from_connection()
         # remove the NM's StateChanged signal receiver
         self.system_bus.remove_signal_receiver(self.connection_state_changed,
                                signal_name='StateChanged',
@@ -1076,13 +1642,15 @@ class DBusInterface(object):
 
     @defer.inlineCallbacks
     def connect(self, do_login=True):
-        """Push the SYS_CONNECT event with the token in the keyring or
-        request the token via oauthdesktop and push the acquired token."""
+        """Push the SYS_USER_CONNECT event with the token.
+
+        The token is in the keyring or it's requested via oauthdesktop.
+        """
         from ubuntuone.syncdaemon.main import NoAccessToken
         try:
             access_token = self.main.get_access_token()
-            self.event_queue.push('SYS_CONNECT', access_token)
-        except NoAccessToken, e:
+            self.event_queue.push('SYS_USER_CONNECT', access_token)
+        except NoAccessToken:
             if do_login:
                 yield self._request_token()
                 self.connect(do_login=False)
@@ -1134,8 +1702,8 @@ class DBusInterface(object):
         return d
 
     def disconnect(self):
-        """ Push the SYS_DISCONNECT event. """
-        self.event_queue.push('SYS_DISCONNECT')
+        """ Push the SYS_USER_DISCONNECT event. """
+        self.event_queue.push('SYS_USER_DISCONNECT')
 
     def quit(self):
         """ calls Main.quit. """
