@@ -15,7 +15,7 @@
 #
 # You should have received a copy of the GNU General Public License along
 # with this program.  If not, see <http://www.gnu.org/licenses/>.
-'''Module that implements the File System Manager.'''
+"""Module that implements the File System Manager."""
 from __future__ import with_statement
 
 import os
@@ -130,14 +130,14 @@ is_forbidden = set("info path node_id share_id is_dir".split()
                   ).intersection
 
 class InconsistencyError(Exception):
-    '''Inconsistency between internal records and filesystem itself.'''
+    """Inconsistency between internal records and filesystem itself."""
 
 class Despair(Exception):
-    '''This should never happen, we're in an impossible condition!'''
+    """This should never happen, we're in an impossible condition!"""
 
 
 class _MDObject(object):
-    '''Wrapper around MD dict.'''
+    """Wrapper around MD dict."""
     def __init__(self, **mdobj):
         self.__dict__.update(mdobj)
 
@@ -150,7 +150,7 @@ class _MDObject(object):
 
 
 class ShareNodeDict(dict):
-    '''Cache for node_id and share.'''
+    """Cache for node_id and share."""
     # pylint: disable-msg=W0612
     def __getitem__(self, key):
         share_id, node_id = key
@@ -191,7 +191,7 @@ class TrashFileShelf(file_shelf.CachedFileShelf):
 
 
 class FileSystemManager(object):
-    '''Keeps the files/dirs metadata and interacts with the filesystem.
+    """Keeps the files/dirs metadata and interacts with the filesystem.
 
     It has a FileShelf where all the metadata is stored, using 'mdid's as
     keys.  'mdid' is 'metadata id'... it's actually an uuid, but we call it
@@ -202,7 +202,7 @@ class FileSystemManager(object):
 
       - idx_path: relationship path -> mdid
       - idx_node_id: relationship (share_id, node_id) -> mdid
-    '''
+    """
 
     CONFLICT_SUFFIX = '.u1conflict'
     CHANGED_LOCAL = 'LOCAL'
@@ -248,11 +248,11 @@ class FileSystemManager(object):
             load_method = getattr(self, "_load_metadata_%s" % md_version)
             load_method(md_version)
 
-        logger("initialized: idx_path: %s, idx_node_id: %s, shares: %s" %
-               (len(self._idx_path), len(self._idx_node_id), len(self.shares)))
+        logger("initialized: idx_path: %s, idx_node_id: %s, shares: %s",
+               len(self._idx_path), len(self._idx_node_id), len(self.shares))
 
     def register_eq(self, eq):
-        '''Registers an EventQueue here.'''
+        """Registers an EventQueue here."""
         self.eq = eq
 
     def _safe_fs_iteritems(self):
@@ -277,9 +277,9 @@ class FileSystemManager(object):
                 # check if the share exists
                 try:
                     self._get_share(mdobj["share_id"])
-                except KeyError, e:
+                except KeyError:
                     # oops, the share is gone!, invalidate this mdid
-                    log_warning('Share %s disappeared! deleting mdid: %s', mdobj['share_id'], mdid)
+                    log_warning('Share %r disappeared! deleting mdid: %s', mdobj['share_id'], mdid)
                     del self.fs[mdid]
                     return False
                 else:
@@ -301,7 +301,7 @@ class FileSystemManager(object):
 
 
     def _load_metadata_None(self, old_version):
-        '''Loads metadata from when it wasn't even versioned.'''
+        """Loads metadata from when it wasn't even versioned."""
         logger("loading metadata from old version %r", old_version)
 
         for mdid, mdobj in self._safe_fs_iteritems():
@@ -338,7 +338,7 @@ class FileSystemManager(object):
             fh.write(METADATA_VERSION)
 
     def _load_metadata_1(self, old_version):
-        '''Loads metadata from version 1.'''
+        """Loads metadata from version 1."""
         logger("loading metadata from old version %r", old_version)
 
         for mdid, mdobj in self._safe_fs_iteritems():
@@ -372,7 +372,7 @@ class FileSystemManager(object):
             fh.write(METADATA_VERSION)
 
     def _load_metadata_2(self, old_version):
-        '''Loads metadata from version 2.'''
+        """Loads metadata from version 2."""
         logger("loading metadata from old version %r", old_version)
 
         for mdid, mdobj in self._safe_fs_iteritems():
@@ -398,7 +398,7 @@ class FileSystemManager(object):
             fh.write(METADATA_VERSION)
 
     def _load_metadata_3(self, old_version):
-        '''Loads metadata from version 3.'''
+        """Loads metadata from version 3."""
         logger("loading metadata from old version %r", old_version)
 
         for mdid, mdobj in self._safe_fs_iteritems():
@@ -417,7 +417,7 @@ class FileSystemManager(object):
             fh.write(METADATA_VERSION)
 
     def _load_metadata_updated(self):
-        '''Loads metadata of last version.'''
+        """Loads metadata of last version."""
         logger("loading updated metadata")
         for mdid, mdobj in self._safe_fs_iteritems():
             abspath = self.get_abspath(mdobj["share_id"], mdobj["path"])
@@ -426,7 +426,7 @@ class FileSystemManager(object):
                 self._idx_node_id[(mdobj["share_id"], mdobj["node_id"])] = mdid
 
     def create(self, path, share_id, node_id=None, is_dir=False):
-        '''Creates a new md object.'''
+        """Creates a new md object."""
         if not path.strip():
             raise ValueError("Empty paths are not allowed (got %r)" % path)
 
@@ -447,8 +447,8 @@ class FileSystemManager(object):
         if node_id is not None:
             self._set_node_id(newobj, node_id, path)
 
-        log_debug("create: path=%r mdid=%r share_id=%r node_id=%r is_dir=%r" \
-                  % (path, mdid, share_id, None, is_dir))
+        log_debug("create: path=%r mdid=%r share_id=%r node_id=%r is_dir=%r",
+                  path, mdid, share_id, None, is_dir)
         self.fs[mdid] = newobj
 
         # adjust the index
@@ -457,7 +457,7 @@ class FileSystemManager(object):
         return mdid
 
     def set_node_id(self, path, node_id):
-        '''Sets the node_id to a md object.'''
+        """Sets the node_id to a md object."""
         path = os.path.normpath(path)
         mdid = self._idx_path[path]
         mdobj = self.fs[mdid]
@@ -470,7 +470,7 @@ class FileSystemManager(object):
             # the object is already there! it's ok if it has the same id
             if mdobj["node_id"] == node_id:
                 log_warning("set_node_id (repeated!): path=%r mdid=%r "
-                            "node_id=%r" % (path, mdobj['mdid'], node_id))
+                            "node_id=%r", path, mdobj['mdid'], node_id)
                 return
             msg = "The path %r already has node_id (%r)" % (path, node_id)
             raise ValueError(msg)
@@ -482,8 +482,8 @@ class FileSystemManager(object):
         mdobj["node_id"] = node_id
         mdobj["info"]["node_id_assigned"] = time.time()
 
-        log_debug("set_node_id: path=%r mdid=%r share_id=%r node_id=%r" % (
-                                     path, mdobj['mdid'], share_id, node_id))
+        log_debug("set_node_id: path=%r mdid=%r share_id=%r node_id=%r",
+                  path, mdobj['mdid'], share_id, node_id)
 
     def get_mdobjs_by_share_id(self, share_id, base_path=None):
         """Returns all the mdobj that belongs to a share and it path
@@ -501,8 +501,20 @@ class FileSystemManager(object):
                     all_mdobjs.append(_MDObject(**mdobj))
         return all_mdobjs
 
+    def get_mdobjs_in_dir(self, base_path):
+        """Return all the mdobjs which dir is base_path."""
+        all_mdobjs = []
+        sep = os.path.sep
+        base_path += sep
+        len_base = len(base_path)
+        for path, mdid in self._idx_path.iteritems():
+            if path[:len_base] == base_path and sep not in path[len_base:]:
+                mdobj = self.fs[mdid]
+                all_mdobjs.append(_MDObject(**mdobj))
+        return all_mdobjs
+
     def get_data_for_server_rescan(self):
-        '''Generates all the (share, node, hash) tuples needed for rescan'''
+        """Generates all the (share, node, hash) tuples needed for rescan"""
         all_data = []
         for _, v in self.fs.items():
             if v['node_id']:
@@ -510,39 +522,55 @@ class FileSystemManager(object):
                         (v['share_id'], v['node_id'], v['server_hash']))
         return all_data
 
+    def get_for_server_rescan_by_path(self, base_path):
+        """
+        Generates all the (share, node, hash) tuples, for the nodes
+        starting with 'path', needed for rescan.
+
+        """
+        all_data = []
+        for path, _ in self.get_paths_starting_with(base_path):
+            mdid = self._idx_path[path]
+            mdobj = self.fs[mdid]
+            if mdobj['node_id']:
+                all_data.append((mdobj['share_id'],
+                                 mdobj['node_id'],
+                                 mdobj['server_hash']))
+        return all_data
+
     def get_by_mdid(self, mdid):
-        '''Returns the md object according to the mdid.'''
+        """Returns the md object according to the mdid."""
         mdobj = self.fs[mdid]
         return _MDObject(**mdobj)
 
     def get_by_path(self, path):
-        '''Returns the md object according to the path.'''
+        """Returns the md object according to the path."""
         path = os.path.normpath(path)
         mdid = self._idx_path[path]
         mdobj = self.fs[mdid]
         return _MDObject(**mdobj)
 
     def get_by_node_id(self, share_id, node_id):
-        '''Returns the md object according to the node_id and share_id.'''
+        """Returns the md object according to the node_id and share_id."""
         mdid = self._idx_node_id[(share_id, node_id)]
         mdobj = self.fs[mdid]
         return _MDObject(**mdobj)
 
     def set_by_mdid(self, mdid, **kwargs):
-        '''Set some values to the md object with that mdid.'''
+        """Set some values to the md object with that mdid."""
         forbidden = is_forbidden(set(kwargs))
         if forbidden:
             raise ValueError("The following attributes can not be set "
                              "externally: %s" % forbidden)
 
-        log_debug("set mdid=%r: %s" % (mdid, kwargs))
+        log_debug("set mdid=%r: %s", mdid, kwargs)
         mdobj = self.fs[mdid]
         for k, v in kwargs.items():
             mdobj[k] = v
         self.fs[mdid] = mdobj
 
     def set_by_path(self, path, **kwargs):
-        '''Set some values to the md object with that path.'''
+        """Set some values to the md object with that path."""
         if "mdid" in kwargs:
             raise ValueError("The mdid is forbidden to set externally")
         path = os.path.normpath(path)
@@ -550,14 +578,14 @@ class FileSystemManager(object):
         self.set_by_mdid(mdid, **kwargs)
 
     def set_by_node_id(self, node_id, share_id, **kwargs):
-        '''Set some values to the md object with that node_id/share_id.'''
+        """Set some values to the md object with that node_id/share_id."""
         if "mdid" in kwargs:
             raise ValueError("The mdid is forbidden to set externally")
         mdid = self._idx_node_id[(share_id, node_id)]
         self.set_by_mdid(mdid, **kwargs)
 
     def refresh_stat(self, path):
-        '''Refreshes the stat of a md object.'''
+        """Refreshes the stat of a md object."""
         log_debug("refresh stat to path=%r", path)
         mdid = self._idx_path[path]
         mdobj = self.fs[mdid]
@@ -565,7 +593,7 @@ class FileSystemManager(object):
         self.fs[mdid] = mdobj
 
     def move_file(self, new_share_id, path_from, path_to):
-        '''Moves a file/dir from one point to the other.'''
+        """Moves a file/dir from one point to the other."""
         path_from = os.path.normpath(path_from)
         path_to = os.path.normpath(path_to)
         mdid = self._idx_path[path_from]
@@ -576,16 +604,17 @@ class FileSystemManager(object):
         to_context = self._enable_share_write(new_share_id, path_to)
 
         # pylint: disable-msg=W0704
+        if mdobj["is_dir"]:
+            expected_event = "FS_DIR_MOVE"
+        else:
+            expected_event = "FS_FILE_MOVE"
         try:
             with contextlib.nested(from_context, to_context):
-                if mdobj["is_dir"]:
-                    expected_event = "FS_DIR_MOVE"
-                else:
-                    expected_event = "FS_FILE_MOVE"
                 self.eq.add_to_mute_filter(expected_event, path_from, path_to)
                 shutil.move(path_from, path_to)
         except IOError, e:
             # file was not yet created
+            self.eq.rm_from_mute_filter(expected_event, path_from, path_to)
             m = "IOError %s when trying to move file/dir %r"
             log_warning(m, e, path_from)
         self.moved(new_share_id, path_from, path_to)
@@ -645,11 +674,11 @@ class FileSystemManager(object):
 
 
     def delete_metadata(self, path):
-        '''Deletes the metadata.'''
+        """Deletes the metadata."""
         path = os.path.normpath(path)
         mdid = self._idx_path[path]
         mdobj = self.fs[mdid]
-        log_debug("delete metadata: path=%r mdid=%r" % (path, mdid))
+        log_debug("delete metadata: path=%r mdid=%r", path, mdid)
 
         # adjust all
         del self._idx_path[path]
@@ -666,42 +695,50 @@ class FileSystemManager(object):
         Notifications are disabled for every sub-path within path.
 
         """
-        dir_deleted = False
-        subtree = self.get_paths_starting_with(path)
-
+        # check metadata to see if any node in LOCAL
+        subtree = self.get_paths_starting_with(path, include_base=False)
         for p, is_dir in subtree:
             if self.changed(path=p) == self.CHANGED_LOCAL:
-                break
-        else:
-            # no local modification in the path tree
-            for p, is_dir in subtree:
-                filter_name = is_dir and "FS_DIR_DELETE" or "FS_FILE_DELETE"
-                self.eq.add_to_mute_filter(filter_name, p)
-                if p != path:
-                    self.delete_metadata(p)
-            shutil.rmtree(path)
-            dir_deleted = True
+                logger("Conflicting dir on remove because %r is local", p)
+                return False
 
-        return dir_deleted
+        # check disk searching for previous conflicts
+        for (dirpath, dirnames, filenames) in os.walk(path):
+            for fname in filenames + dirnames:
+                if fname.endswith(self.CONFLICT_SUFFIX):
+                    logger("Conflicting dir on remove because of previous "
+                           "conflict on: %r", os.path.join(dirpath, fname))
+                    return False
+
+        # all green
+        for p, is_dir in subtree:
+            filter_name = is_dir and "FS_DIR_DELETE" or "FS_FILE_DELETE"
+            self.eq.add_to_mute_filter(filter_name, p)
+            self.delete_metadata(p)
+        shutil.rmtree(path)
+        return True
 
     def delete_file(self, path):
-        '''Deletes a file/dir and the metadata.'''
+        """Deletes a file/dir and the metadata."""
         # adjust the metadata
         path = os.path.normpath(path)
         mdid = self._idx_path[path]
         mdobj = self.fs[mdid]
-        log_debug("delete: path=%r mdid=%r" % (path, mdid))
+        log_debug("delete: path=%r mdid=%r", path, mdid)
 
         # delete the file
+        if self.is_dir(path=path):
+            expected_event = "FS_DIR_DELETE"
+            remove_action = os.rmdir
+        else:
+            expected_event = "FS_FILE_DELETE"
+            remove_action = os.remove
         try:
             with self._enable_share_write(mdobj['share_id'], path):
-                if self.is_dir(path=path):
-                    self.eq.add_to_mute_filter("FS_DIR_DELETE", path)
-                    os.rmdir(path)
-                else:
-                    self.eq.add_to_mute_filter("FS_FILE_DELETE", path)
-                    os.remove(path)
+                self.eq.add_to_mute_filter(expected_event, path)
+                remove_action(path)
         except OSError, e:
+            self.eq.rm_from_mute_filter(expected_event, path)
             if e.errno == errno.ENOTEMPTY:
                 if not self._delete_dir_tree(path=path):
                     raise
@@ -711,24 +748,25 @@ class FileSystemManager(object):
         self.delete_metadata(path)
 
     def move_to_conflict(self, mdid):
-        '''Moves a file/dir to its .conflict.'''
+        """Moves a file/dir to its .conflict."""
         mdobj = self.fs[mdid]
         path = self.get_abspath(mdobj['share_id'], mdobj['path'])
-        log_debug("move_to_conflict: path=%r mdid=%r" % (path, mdid))
+        log_debug("move_to_conflict: path=%r mdid=%r", path, mdid)
         base_to_path = to_path =  path + self.CONFLICT_SUFFIX
         ind = 0
         while os.path.exists(to_path):
             ind += 1
             to_path = base_to_path + "." + str(ind)
+        if mdobj["is_dir"]:
+            expected_event = "FS_DIR_MOVE"
+        else:
+            expected_event = "FS_FILE_MOVE"
         with self._enable_share_write(mdobj['share_id'], path):
             try:
-                if mdobj["is_dir"]:
-                    expected_event = "FS_DIR_MOVE"
-                else:
-                    expected_event = "FS_FILE_MOVE"
                 self.eq.add_to_mute_filter(expected_event, path, to_path)
                 os.rename(path, to_path)
             except OSError, e:
+                self.eq.rm_from_mute_filter(expected_event, path, to_path)
                 if e.errno == errno.ENOENT:
                     m = "Already removed when trying to move to conflict: %r"
                     log_warning(m, path)
@@ -737,7 +775,7 @@ class FileSystemManager(object):
 
         # search for given path plus the separator to avoid self, and
         # other similar paths that aren't *children*
-        for p, is_dir in self.get_paths_starting_with(path + os.path.sep):
+        for p, is_dir in self.get_paths_starting_with(path, include_base=False):
             if is_dir:
                 # remove inotify watch
                 try:
@@ -754,7 +792,7 @@ class FileSystemManager(object):
         self.fs[mdid] = mdobj
 
     def _check_partial(self, mdid):
-        '''Checks consistency between internal flag and FS regarding partial'''
+        """Checks consistency between internal flag and FS regarding partial"""
         # get the values
         mdobj = self.fs[mdid]
         partial_in_md = mdobj["info"]["is_partial"]
@@ -768,21 +806,20 @@ class FileSystemManager(object):
         return partial_in_md
 
     def _get_partial_path(self, mdobj):
-        '''Gets the path of the .partial file for a given mdobj'''
-        is_dir = mdobj["is_dir"]
+        """Gets the path of the .partial file for a given mdobj"""
         path = self.get_abspath(mdobj['share_id'], mdobj['path'])
         partial_path = os.path.join(self.partials_dir, mdobj['mdid'] + '.u1partial')
         dirname, filename = os.path.split(path)
         return partial_path + '.' + filename
 
     def create_partial(self, node_id, share_id):
-        '''Creates a .partial in disk and set the flag in metadata.'''
+        """Creates a .partial in disk and set the flag in metadata."""
         mdid = self._idx_node_id[(share_id, node_id)]
-        log_debug("create_partial: mdid=%r share_id=%r node_id=%r" %
-                  (mdid, share_id, node_id))
+        log_debug("create_partial: mdid=%r share_id=%r node_id=%r",
+                  mdid, share_id, node_id)
         if self._check_partial(mdid):
             raise ValueError("The object with share_id %r and node_id %r is "
-                             "already partial!" % (share_id, node_id))
+                             "already partial!", share_id, node_id)
 
         # create an empty partial and set the flag
         mdobj = self.fs[mdid]
@@ -802,7 +839,7 @@ class FileSystemManager(object):
         self.fs[mdid] = mdobj
 
     def get_partial_for_writing(self, node_id, share_id):
-        '''Get a write-only fd to a partial file'''
+        """Get a write-only fd to a partial file"""
         mdid = self._idx_node_id[(share_id, node_id)]
         log_debug("get_partial_for_writing: mdid=%r share_id=%r node_id=%r" %
                   (mdid, share_id, node_id))
@@ -815,7 +852,7 @@ class FileSystemManager(object):
         return open(partial_path, "w")
 
     def get_partial(self, node_id, share_id):
-        '''Gets a read-only fd to a partial file.'''
+        """Gets a read-only fd to a partial file."""
         mdid = self._idx_node_id[(share_id, node_id)]
         if not self._check_partial(mdid):
             raise ValueError("The object with share_id %r and node_id %r is "
@@ -826,7 +863,7 @@ class FileSystemManager(object):
         return fd
 
     def commit_partial(self, node_id, share_id, local_hash):
-        '''Creates a .partial in disk and set the flag in metadata.'''
+        """Creates a .partial in disk and set the flag in metadata."""
         mdid = self._idx_node_id[(share_id, node_id)]
         mdobj = self.fs[mdid]
         if mdobj["is_dir"]:
@@ -837,8 +874,8 @@ class FileSystemManager(object):
 
         # move the .partial to the real path, and set the md info
         path = self.get_abspath(mdobj['share_id'], mdobj['path'])
-        log_debug("commit_partial: path=%r mdid=%r share_id=%r node_id=%r" %
-                                            (path, mdid, share_id, node_id))
+        log_debug("commit_partial: path=%r mdid=%r share_id=%r node_id=%r",
+                  path, mdid, share_id, node_id)
 
         partial_path = self._get_partial_path(mdobj)
         with self._enable_share_write(share_id, path):
@@ -852,14 +889,14 @@ class FileSystemManager(object):
         self.fs[mdid] = mdobj
 
     def remove_partial(self, node_id, share_id):
-        '''Removes a .partial in disk and set the flag in metadata.'''
+        """Removes a .partial in disk and set the flag in metadata."""
         mdid = self._idx_node_id[(share_id, node_id)]
 
         # delete the .partial, and set the md info
         mdobj = self.fs[mdid]
         path = self.get_abspath(mdobj['share_id'], mdobj['path'])
-        log_debug("remove_partial: path=%r mdid=%r share_id=%r node_id=%r" %
-                                            (path, mdid, share_id, node_id))
+        log_debug("remove_partial: path=%r mdid=%r share_id=%r node_id=%r",
+                  path, mdid, share_id, node_id)
         partial_path = self._get_partial_path(mdobj)
         #pylint: disable-msg=W0704
         try:
@@ -874,14 +911,14 @@ class FileSystemManager(object):
         self.fs[mdid] = mdobj
 
     def upload_finished(self, mdid, server_hash):
-        '''Sets the metadata with timestamp and server hash.'''
+        """Sets the metadata with timestamp and server hash."""
         mdobj = self.fs[mdid]
         mdobj["info"]["last_uploaded"] = time.time()
         mdobj["server_hash"] = server_hash
         self.fs[mdid] = mdobj
 
     def _get_mdid_from_args(self, kwargs, parent):
-        '''Parses the kwargs and gets the mdid.'''
+        """Parses the kwargs and gets the mdid."""
         if len(kwargs) == 1 and "path" in kwargs:
             path = os.path.normpath(kwargs["path"])
             return self._idx_path[path]
@@ -892,13 +929,13 @@ class FileSystemManager(object):
         raise TypeError("Incorrect arguments for %r: %r" % (parent, kwargs))
 
     def is_dir(self, **kwargs):
-        '''Return True if the path of a given object is a directory.'''
+        """Return True if the path of a given object is a directory."""
         mdid = self._get_mdid_from_args(kwargs, "is_dir")
         mdobj = self.fs[mdid]
         return mdobj["is_dir"]
 
     def has_metadata(self, **kwargs):
-        '''Return True if there's metadata for a given object.'''
+        """Return True if there's metadata for a given object."""
         if len(kwargs) == 1 and "path" in kwargs:
             path = os.path.normpath(kwargs["path"])
             return path in self._idx_path
@@ -912,15 +949,15 @@ class FileSystemManager(object):
         """Return whether a given node has changed or not.
 
         The node can be defined by any of the following:
-         * path
-         * metadata's id (mdid)
-         * node_id and share_id
+            - path
+            - metadata's id (mdid)
+            - node_id and share_id
 
         Return:
-         * LOCAL if the node has local modifications that the server is
-         not aware of.
-         * SERVER if the node is not fully downloaded.
-         * NONE the node has not changed.
+            - LOCAL if the node has local modifications that the server is
+              not aware of.
+            - SERVER if the node is not fully downloaded.
+            - NONE the node has not changed.
 
         """
         # get the values
@@ -958,7 +995,7 @@ class FileSystemManager(object):
         return has_changed
 
     def dir_content(self, path):
-        '''Returns the content of the directory in a server-comparable way.'''
+        """Returns the content of the directory in a server-comparable way."""
         path = os.path.normpath(path)
         mdid = self._idx_path[path]
         mdobj = self.fs[mdid]
@@ -966,7 +1003,7 @@ class FileSystemManager(object):
             raise ValueError("You can ask dir_content only on a directory.")
 
         def _get_all():
-            '''find the mdids that match'''
+            """find the mdids that match"""
             for p, m in self._idx_path.iteritems():
                 if os.path.dirname(p) == path and p != path:
                     mdobj = self.fs[m]
@@ -976,7 +1013,7 @@ class FileSystemManager(object):
         return sorted(_get_all())
 
     def open_file(self, mdid):
-        '''returns a file like object for reading the contents of the file.'''
+        """returns a file like object for reading the contents of the file."""
         mdobj = self.fs[mdid]
         if mdobj["is_dir"]:
             raise ValueError("You can only open files, not directories.")
@@ -997,12 +1034,13 @@ class FileSystemManager(object):
         # remove the initial os.path.sep
         return relpath.lstrip(os.path.sep)
 
-    def _get_share(self, share_id):
-        """ returns the share with id: share_id. """
-        share = self.shares.get(share_id, None)
+    def _get_share(self, id):
+        """ returns the share/udf with share or volume id: id. """
+        # TODO: refactor fsm to use volume instead of share
+        share = self.shares.get(id, None)
         if share is None:
-            share = self.vm.shares[share_id]
-            self.shares[share_id] = share
+            share = self.vm.get_volume(id)
+            self.shares[id] = share
         return share
 
     def get_abspath(self, share_id, path):
@@ -1020,38 +1058,48 @@ class FileSystemManager(object):
         return EnableShareWrite(share, path)
 
     def get_paths_starting_with(self, base_path, include_base=True):
-        """ return a list of paths that starts with: path. """
-        len_func = str.__len__ # avoid lookup
-        base_length = len_func(base_path)
+        """Return a list of paths that are starts with base_path.
 
+        base_path should be a directory.
+        If include_base, base_path is added to the resulting list.
+
+        """
         all_paths = []
-        for path in self._idx_path:
-            candidate = include_base or len_func(path) > base_length
-            if candidate and path.startswith(base_path):
-                mdid = self._idx_path[path]
+
+        base_mdid = self._idx_path.get(base_path)
+        if base_mdid is not None and include_base:
+            mdobj = self.fs[base_mdid]
+            all_paths.append((base_path, mdobj['is_dir']))
+
+        # add sep, to always match children in the tree and not partial names
+        base_path += os.path.sep
+
+        for path, mdid in self._idx_path.iteritems():
+            if path.startswith(base_path):
                 mdobj = self.fs[mdid]
                 all_paths.append((path, mdobj['is_dir']))
+
         return all_paths
 
     def delete_to_trash(self, mdid, parent_id):
-        '''Move the node to the trash.'''
+        """Move the node to the trash."""
         mdobj = self.fs[mdid]
         node_id = mdobj["node_id"]
         share_id = mdobj["share_id"]
         path = self.get_abspath(mdobj['share_id'], mdobj['path'])
-        log_debug("delete_to_trash: mdid=%s, parent=%s, share=%s, node=%s, "
+        log_debug("delete_to_trash: mdid=%r, parent=%r, share=%r, node=%r, "
                   "path=%r", mdid, parent_id, share_id, node_id, path)
         self.delete_metadata(path)
         self.trash[(share_id, node_id)] = (mdid, parent_id)
 
     def remove_from_trash(self, share_id, node_id):
-        '''Deletes the node from the trash.'''
-        log_debug("remove_from_trash: share=%s, node=%s", share_id, node_id)
+        """Deletes the node from the trash."""
+        log_debug("remove_from_trash: share=%r, node=%r", share_id, node_id)
         if (share_id, node_id) in self.trash:
             del self.trash[(share_id, node_id)]
 
     def get_iter_trash(self):
-        '''Return the trash element by element.'''
+        """Return the trash element by element."""
         for (share_id, node_id), (mdid, parent_id) in self.trash.iteritems():
             yield share_id, node_id, parent_id
 
