@@ -134,10 +134,10 @@ class StateMachineRunner(object):
             self.log.debug("Calling %s"%action_func_name)
             try:
                 af(event_name, parameters, *args)
-            except Exception:
+            except Exception, e:
                 self.log.exception("Executing ACTION_FUNC '%s' "
-                                   "gave an exception" %
-                                   (action_func_name))
+                                   "gave an exception: %r" %
+                                   (action_func_name, e))
                 self.on_error(event_name, parameters)
                 return
         # validate the end state
