@@ -15,23 +15,27 @@
 #
 # You should have received a copy of the GNU General Public License along
 # with this program.  If not, see <http://www.gnu.org/licenses/>.
-'''Module that implements the Local Rescan.'''
+"""Module that implements the Local Rescan."""
 
 import collections
-import os
+import errno
 import functools
 import logging
-import errno
+import os
 import stat
 
+from ubuntuone.syncdaemon import volume_manager
 from ubuntuone.syncdaemon.filesystem_manager import get_stat
 from twisted.internet import defer, reactor
 
+from ubuntuone.syncdaemon.interfaces import IMarker
+
+
 class ScanTransactionDirty(Exception):
-    '''The transaction was dirty.'''
+    """The transaction was dirty."""
 
 class ScanNoDirectory(Exception):
-    '''The whole directory went away.'''
+    """The whole directory went away."""
 
 # local rescan logger
 lr_logger = logging.getLogger('ubuntuone.SyncDaemon.local_rescan')
@@ -41,10 +45,10 @@ log_error = functools.partial(lr_logger.log, logging.ERROR)
 log_warning = functools.partial(lr_logger.log, logging.WARNING)
 
 class LocalRescan(object):
-    '''Local re-scanner.
+    """Local re-scanner.
 
     Compares the real disc with FSM's metadata, and pushes the changes to EQ.
-    '''
+    """
     def __init__(self, vm, fsm, eq, aq):
         self.vm = vm
         self.fsm = fsm
@@ -54,49 +58,59 @@ class LocalRescan(object):
         self._previous_deferred = None
 
     def start(self):
-        '''Start the comparison.'''
-        log_info("start scan all shares")
-        to_scan = self._get_shares()
-        for share in to_scan:
+        """Start the comparison."""
+        log_info("start scan all volumes")
+        to_scan = self._get_volumes(all_volumes=False)
+        for vol in to_scan:
             try:
-                mdobj = self.fsm.get_by_path(share.path)
+                mdobj = self.fsm.get_by_path(vol.path)
             except KeyError:
                 # this could happen in a strange corruption situation where FSM
                 # lost the share information, so we remove it, because VM will
                 # download it again
-                self.vm.share_deleted(share.id)
+                self.vm.share_deleted(vol.volume_id)
             else:
-                self._queue.appendleft((share, share.path, True, mdobj.mdid))
+                self._queue.appendleft((vol, vol.path, True,
+                                        mdobj.mdid, False))
         d = self._queue_scan()
         d.addCallback(self._process_trash)
         return d
 
     def _process_trash(self, _):
-        '''Process the trash of FSM and send AQ orders to unlink the nodes.'''
+        """Process the trash of FSM and send AQ orders to unlink the nodes."""
         log_info("processing trash")
         for share_id, node_id, parent_id in self.fsm.get_iter_trash():
-            log_debug("unlink from trash: share_id: %s   parent_id: %s   "
-                      "node_id: %s",share_id, parent_id, node_id)
+            datalog = "share_id: %r   parent_id: %r   node_id: %r" % (
+                      share_id, parent_id, node_id)
+            if node_id is None or IMarker.providedBy(parent_id):
+                # situation where the node is not in the server
+                log_info("removing from trash: " + datalog)
+                self.fsm.remove_from_trash(share_id, node_id)
+                continue
+            log_info("unlink from trash: " + datalog)
             self.aq.unlink(share_id, parent_id, node_id)
 
-    def _get_shares(self, access_level="Modify"):
-        '''Get all the shares to compare.'''
-        for sid in self.vm.shares:
-            share = self.vm.shares[sid]
-            if share.access_level == access_level:
-                yield share
+    def _get_volumes(self, all_volumes=True, access_level='Modify'):
+        """Get all the shares and udfs matching access_level to compare."""
+        modify = access_level == 'Modify'
+        view = access_level == 'View'
+        for volume in self.vm.get_volumes(all_volumes=all_volumes):
+            if (modify and volume.can_write()) or \
+               (view and not volume.can_write()):
+                yield volume
 
-    def scan_dir(self, mdid, direct):
-        '''Compares one directory between metadata and disk.'''
+    def scan_dir(self, mdid, direct, udfmode=False):
+        """Compares one directory between metadata and disk."""
         log_info("scan dir: %r  mdid: %s", direct, mdid)
 
         # get the share to get only a subset of mdids
-        for share in self._get_shares():
+        for share in self._get_volumes():
             if direct.startswith(share.path):
                 break
         else:
             # not in RW shares; let's check RO shares, otherwise it's an error
-            for share in self._get_shares("View"):
+
+            for share in self._get_volumes(access_level="View"):
                 if direct.startswith(share.path):
                     return
             log_error("The received path is not in any share!")
@@ -110,42 +124,84 @@ class LocalRescan(object):
             # it's better to delay the rescan some miliseconds, as if a
             # directory was moved, it's better to leave stuff some time to
             # settle down
-            reactor.callLater(.1, self._send_scan_error, mdid)
+            reactor.callLater(.1, self._send_scan_error, mdid, udfmode)
             return
         elif not stat.S_ISDIR(stat_result.st_mode):
             m = "The path is in disk but it's not a dir: %r" % direct
             log_error(m)
-            raise ValueError("m")
+            raise ValueError(m)
 
         # No, 'share' is surely defined; pylint: disable-msg=W0631
-        self._queue.appendleft((share, direct, False, mdid))
+        self._queue.appendleft((share, direct, False, mdid, udfmode))
         return self._queue_scan()
 
-    def _send_scan_error(self, mdid):
-        '''Sends the scan error event.'''
-        self.eq.push("LR_SCAN_ERROR", mdid)
+    def _send_scan_error(self, mdid, udfmode):
+        """Sends the scan error event."""
+        self.eq.push("LR_SCAN_ERROR", mdid, udfmode)
 
     def _queue_scan(self):
-        '''If there's a scan in progress, queue the new one for later.'''
+        """If there's a scan in progress, queue the new one for later."""
         if self._previous_deferred is None:
             self._previous_deferred = defer.Deferred()
             self._process_next_queue(None)
         return self._previous_deferred
 
+    def _add_watches_to_udf_ancestors(self, volume):
+        """Add a inotify watch to volume's ancestors if it's an UDF."""
+        added_watches = []
+
+        def revert_watches():
+            """Restore the just added watches and unsubscribe volume."""
+            for path in added_watches:
+                self.eq.inotify_rm_watch(path)
+            m = "Unsubscribing UDF %r because not in disk: %r"
+            log_info(m, volume.volume_id, volume.path)
+            self.vm.unsubscribe_udf(volume.volume_id)
+
+        for ancestor in volume.ancestors:
+            # check that ancestor is still there
+            if not os.access(ancestor, os.R_OK):
+                log_info("Tree broken at path: %r", volume.path)
+                revert_watches()
+                return False
+
+            # add watch if needed
+            log_debug("Adding watch to UDF's ancestor %r", ancestor)
+            if not self.eq.inotify_has_watch(ancestor):
+                added_watches.append(ancestor)
+                self.eq.inotify_add_watch(ancestor)
+
+        # finally, check that UDF is ok in disk
+        if not os.access(volume.path, os.R_OK):
+            revert_watches()
+            return False
+
+        # all is ok
+        return True
+
     def _process_next_queue(self, _):
-        '''Process the next item in the queue, if any.'''
+        """Process the next item in the queue, if any."""
         log_debug("process next in queue (len %d)", len(self._queue))
         if not self._queue:
-            self._previous_deferred.callback(None)
+            d = self._previous_deferred
             self._previous_deferred = None
+            d.callback(None)
             return
 
         # more to scan
         scan_info = self._queue.pop()
 
         def safe_scan():
-            '''Scan safely'''
+            """Scan safely"""
             try:
+                # add watches to UDF ancestors and check UDF is ok
+                volume = scan_info[0]
+                if isinstance(volume, volume_manager.UDF):
+                    udf_ok = self._add_watches_to_udf_ancestors(volume)
+                    if not udf_ok:
+                        self._process_next_queue(None)
+                        return
+
                 self._scan_tree(*scan_info)
             # pylint: disable-msg=W0703
             except Exception, e:
@@ -153,32 +209,34 @@ class LocalRescan(object):
 
         reactor.callLater(0, safe_scan)
 
-    def _get_share_info(self, share_id, path):
-        '''Get all the objects information for a share.'''
+    def _get_share_info(self, path):
+        """Get all the objects information for a directory path."""
         share_info = []
-        for obj in self.fsm.get_mdobjs_by_share_id(share_id, path):
+        for obj in self.fsm.get_mdobjs_in_dir(path):
             changd = self.fsm.changed(mdid=obj.mdid)
             share_info.append((obj.path, obj.is_dir, obj.stat, changd,
                                obj.node_id, obj.local_hash, obj.server_hash))
         return share_info
 
-    def _scan_tree(self, share, path, scan_partials, mdid):
-        '''Scans a whole tree, using the received path as root.'''
+    def _scan_tree(self, share, path, scan_partials, mdid, udfmode):
+        """Scans a whole tree, using the received path as root."""
         log_debug("_scan_tree:  share_path: %r  path: %r", share.path, path)
 
         def go_deeper(newdirs):
-            '''Explore into the subdirs.'''
+            """Explore into the subdirs."""
             for direct in newdirs:
                 log_debug("explore subdir: %r", direct)
-                self._queue.appendleft((share, direct, scan_partials, mdid))
+                self._queue.appendleft((share, direct, scan_partials,
+                                        mdid, udfmode))
 
         def re_launch(failure):
-            '''Explore that directory again.'''
+            """Explore that directory again."""
             if failure.check(ScanTransactionDirty):
                 reason = failure.getErrorMessage()
                 log_debug("re queue, transaction dirty for %r, reason: %s",
                                                                   path, reason)
-                self._queue.appendleft((share, path, scan_partials, mdid))
+                self._queue.appendleft((share, path, scan_partials,
+                                        mdid, udfmode))
             elif failure.check(OSError, IOError):
                 reason = failure.getErrorMessage()
                 m = "Disk error while scanning path %r, reason: %s"
@@ -188,42 +246,46 @@ class LocalRescan(object):
                 # it's better to delay the rescan some miliseconds, as if a
                 # directory was moved, it's better to leave stuff some time to
                 # settle down
-                reactor.callLater(.1, self._send_scan_error, mdid)
+                reactor.callLater(.1, self._send_scan_error, mdid, udfmode)
             else:
                 log_error("in the scan: %s (%s)\n%s",
                           failure.type, failure.value, failure.getTraceback())
                 return failure
 
-        d = defer.succeed((share, path, scan_partials))
+        d = defer.succeed((share, path, scan_partials, udfmode))
         d.addCallbacks(self._scan_one_dir)
         d.addCallbacks(go_deeper, re_launch)
         d.addCallback(self._process_next_queue)
         return d
 
     def _compare(self, dirpath, dirnames, filenames, share, scan_partials):
-        '''Compare the directories with the info that should be there.'''
+        """Compare the directories with the info that should be there."""
         log_debug("comparing directory %r", dirpath)
 
         # get the share info
-        share_info = self._get_share_info(share.id, dirpath)
+        share_info = self._get_share_info(dirpath)
         shouldbe = self._paths_filter(share_info, dirpath, len(share.path))
 
         def despair(message, fullname, also_children=False, also_remove=None):
-            '''Something went very bad with this node, converge!'''
-            log_debug(message, fullname)
-            os.rename(fullname, fullname + ".u1conflict")
-            self.fsm.delete_metadata(fullname)
-
+            """Something went very bad with this node, converge!"""
             # if asked, remove metadata por children
             if also_children:
-                log_debug("Removing also metadata from %r children", fullname)
+                log_debug("Removing metadata for %r children", fullname)
                 # pylint: disable-msg=W0612
-                for path, is_dir in self.fsm.get_paths_starting_with(fullname):
+                children = self.fsm.get_paths_starting_with(fullname, False)
+                for path, is_dir in children:
                     self.fsm.delete_metadata(path)
 
+            # remove fullname after removing its children,
+            # otherwise metadata removal may fail
+            log_info(message, fullname)
+            os.rename(fullname, fullname + ".u1conflict")
+            self.fsm.delete_metadata(fullname)
+
             # if asked, remove also that file (if still exists)
             if also_remove is not None:
                 try:
+                    log_info("Also remove %r", also_remove)
                     os.remove(also_remove)
                 except OSError, e:
                     if e.errno != errno.ENOENT:
@@ -231,11 +293,10 @@ class LocalRescan(object):
 
 
         def check_stat(fullname, statinfo):
-            '''Generate event if stats differ.'''
-            log_debug("comp yield STAT prv: %s", statinfo)
+            """Generate event if stats differ."""
             newstat = os.stat(fullname)
-            log_debug("comp yield STAT new: %s", newstat)
             if statinfo != newstat:
+                log_debug("stat differ for: %r", fullname)
                 events.append(('FS_FILE_CLOSE_WRITE', fullname))
 
         # check all directories
@@ -244,10 +305,10 @@ class LocalRescan(object):
         for dname in dirnames:
             fullname = os.path.join(dirpath, dname)
             if dname in shouldbe:
-                is_dir, statinfo, changed, _ = shouldbe.pop(dname)
+                is_dir, statinfo, changed = shouldbe.pop(dname)
                 if not is_dir:
                     # it's there, but it's a file!
-                    log_debug("comp yield: file %r became a dir!", fullname)
+                    log_info("comp yield: file %r became a dir!", fullname)
                     events.append(('FS_FILE_DELETE', fullname))
                     events.append(('FS_DIR_CREATE', fullname))
                 else:
@@ -261,7 +322,7 @@ class LocalRescan(object):
                         to_scan_later.append(fullname)
                     elif changed == "NONE":
                         # it's old, we should scan it later
-                        log_debug("comp yield: dir %r will be scaned later!",
+                        log_debug("comp yield: dir %r will be scaned later because is in NONE!",
                                                                       fullname)
                         to_scan_later.append(fullname)
                     else:
@@ -277,9 +338,9 @@ class LocalRescan(object):
         for fname in filenames:
             fullname = os.path.join(dirpath, fname)
             if fname in shouldbe:
-                is_dir, statinfo, changed, _ = shouldbe.pop(fname)
+                is_dir, statinfo, changed = shouldbe.pop(fname)
                 if is_dir:
-                    log_debug("comp yield: dir %r became a file!", fullname)
+                    log_info("comp yield: dir %r became a file!", fullname)
                     # it's there, but it's a directory!
                     events.append(('FS_DIR_DELETE', fullname))
                     events.append(('FS_FILE_CREATE', fullname))
@@ -292,7 +353,7 @@ class LocalRescan(object):
                         events.append(('FS_FILE_CLOSE_WRITE', fullname))
                     elif changed == "NONE":
                         # what about stat info?
-                        log_debug("comp yield: file %r was here.. stat?",
+                        log_debug("comp yield: file %r was here, let's do check_stat",
                                                                     fullname)
                         check_stat(fullname, statinfo)
                     elif changed == "SERVER":
@@ -327,7 +388,7 @@ class LocalRescan(object):
                         os.remove(fullname)
                         continue
 
-                    is_dir, statinfo, changed, _ = shouldbe.pop(realname)
+                    is_dir, statinfo, changed = shouldbe.pop(realname)
                     if is_dir:
                         m = ".partial of a file that MD says it's a dir: %r"
                         despair(m, realfullname, also_remove=fullname)
@@ -394,7 +455,7 @@ class LocalRescan(object):
                 despair(m, realfullname, also_remove=partial_path)
                 continue
 
-            is_dir, statinfo, changed, _ = shouldbe.pop(realname)
+            is_dir, statinfo, changed = shouldbe.pop(realname)
             if is_dir:
                 m = ".partial of a file that MD says it's a dir: %r"
                 despair(m, realfullname, also_remove=partial_path)
@@ -410,7 +471,7 @@ class LocalRescan(object):
                 self.fsm.remove_partial(mdobj.node_id, mdobj.share_id)
 
         # all these don't exist anymore
-        for name, (is_dir, statinfo, changed, lhash) in shouldbe.iteritems():
+        for name, (is_dir, statinfo, changed) in shouldbe.iteritems():
             fullname = os.path.join(dirpath, name)
             if is_dir:
                 if changed not in ("SERVER", "NONE"):
@@ -422,16 +483,16 @@ class LocalRescan(object):
                         self.fsm.delete_metadata(path)
                     continue
 
-                log_debug("comp yield: directory %r is gone!", fullname)
+                log_info("comp yield: directory %r is gone!", fullname)
                 # it's a directory, didn't have any info inside?
-                base_path = fullname[len(share.path)+1:]
                 to_inform = []
 
                 # get all the info inside that dir
-                for shrpath, is_dir, statinfo, _, _, _, _ in share_info:
-                    if shrpath.startswith(base_path):
-                        qparts = len(shrpath.split(os.path.sep))
-                        to_inform.append((qparts, shrpath, is_dir))
+                objs = self.fsm.get_mdobjs_by_share_id(share.volume_id, fullname)
+                for obj in objs:
+                    shrpath = obj.path
+                    qparts = len(shrpath.split(os.path.sep))
+                    to_inform.append((qparts, shrpath, obj.is_dir))
 
                 # order everything from more path components to less (this
                 # will assure correct upgoing walk in the tree)
@@ -440,63 +501,69 @@ class LocalRescan(object):
                 # inform deletion!
                 for (_, name, is_dir) in to_inform:
                     fullname = os.path.join(share.path, name)
+                    log_info("Inform deletion of stuff in dir: %r", fullname)
                     if is_dir:
                         events.append(('FS_DIR_DELETE', fullname))
                     else:
                         events.append(('FS_FILE_DELETE', fullname))
             else:
-                if changed not in ("SERVER", "NONE", "LOCAL"):
+                if changed == 'SERVER':
+                    # download interruped and partial lost
+                    log_debug("comp yield: file %r not in disk, in SERVER "
+                              "state, without partial!", fullname)
+                    mdobj = self.fsm.get_by_path(fullname)
+                    self.fsm.set_by_mdid(mdobj.mdid,
+                                         server_hash=mdobj.local_hash)
+                    self.fsm.remove_partial(mdobj.node_id, mdobj.share_id)
+                elif changed in ('NONE', 'LOCAL'):
+                    # if it had content somewhen, now is really gone (otherwise
+                    # it was never really created in the disk)
+                    log_info("comp yield: file %r is gone!", fullname)
+                    events.append(('FS_FILE_DELETE', fullname))
+                else:
                     # bad metadata
                     m = "Bad 'changed': removing MD from file %r"
                     log_debug(m, fullname)
                     self.fsm.delete_metadata(fullname)
-                    continue
 
-                # if it had content somewhen, now is gone (otherwise it was
-                # never really created in the disk)
-                if lhash:
-                    log_debug("comp yield: file %r is gone!", fullname)
-                    events.append(('FS_FILE_DELETE', fullname))
         return events, to_scan_later
 
     def _paths_filter(self, shrinfo, dirpath, len_shr_path):
-        '''Returns the paths that belong to this dir.'''
+        """Returns the paths that belong to this dir."""
         # paths in shares are relative, remove the first slash
-        direct = dirpath[len_shr_path + 1:]
         basedir = dirpath[:len_shr_path]
 
         # build the dict
         filesdirs = {}
         for fpath, is_dir, statinfo, changed, node_id, lhash, shash in shrinfo:
-            base, fname = os.path.split(fpath)
-            if base == direct:
-                # if without node_id, remove the metadata, and take it as new
-                if node_id is None:
-                    fullname = os.path.join(basedir, fpath)
-                    m = "Deleting metadata, because of node_id=None, of %r"
-                    log_debug(m, fullname)
-                    self.fsm.delete_metadata(fullname)
+            fname = os.path.basename(fpath)
+            # if without node_id, remove the metadata, and take it as new
+            if node_id is None:
+                fullname = os.path.join(basedir, fpath)
+                m = "Deleting metadata, because of node_id=None, of %r"
+                log_debug(m, fullname)
+                self.fsm.delete_metadata(fullname)
 
-                # if both hashes aren't set in a file, it's a non-content
-                # situation, remove the metadata
-                elif not is_dir and not lhash and not shash:
-                    fullname = os.path.join(basedir, fpath)
-                    m = "Deleting metadata, both hashes empty, of %r"
-                    log_debug(m, fullname)
-                    self.fsm.delete_metadata(fullname)
+            # if both hashes aren't set in a file, it's a non-content
+            # situation, remove the metadata
+            elif not is_dir and not lhash and not shash:
+                fullname = os.path.join(basedir, fpath)
+                m = "Deleting metadata, both hashes empty, of %r"
+                log_debug(m, fullname)
+                self.fsm.delete_metadata(fullname)
 
-                    # also set the parent hashes to "", to force a new scan
-                    parent = os.path.dirname(fullname)
-                    log_debug("Dirtying the parent hashes, path: %r", parent)
-                    self.fsm.set_by_path(parent, server_hash="", local_hash="")
+                # also set the parent hashes to "", to force a new scan
+                parent = os.path.dirname(fullname)
+                log_debug("Dirtying the parent hashes, path: %r", parent)
+                self.fsm.set_by_path(parent, server_hash="", local_hash="")
 
-                else:
-                    filesdirs[fname] = is_dir, statinfo, changed, lhash
+            else:
+                filesdirs[fname] = is_dir, statinfo, changed
         return filesdirs
 
     def _scan_one_dir(self, scan_info):
-        '''Gets one dir and compares with fsm.'''
-        share, dirpath, partials = scan_info
+        """Gets one dir and compares with fsm."""
+        share, dirpath, partials, udf_mode = scan_info
 
         if self.eq.inotify_has_watch(dirpath):
             log_debug("Path already has watch: %r", dirpath)
@@ -508,7 +575,8 @@ class LocalRescan(object):
         self.eq.freeze_begin(dirpath)
 
         def scan():
-            '''the scan, really'''
+            """The scan, really."""
+
             log_debug("scanning the dir %r", dirpath)
             listdir = os.listdir(dirpath)
 
@@ -521,6 +589,10 @@ class LocalRescan(object):
                 if stat_result is None or \
                    stat.S_ISLNK(stat_result.st_mode):
                     continue
+                elif not os.access(fullname, os.R_OK):
+                    log_warning("Ignoring path as we don't have enough "
+                                "permissions to track it: %r", fullname)
+                    continue
                 elif stat.S_ISDIR(stat_result.st_mode):
                     dnames.append(something)
                 elif stat.S_ISREG(stat_result.st_mode):
@@ -534,15 +606,45 @@ class LocalRescan(object):
             to_later.extend(to_scan_later)
             return events
 
+        delete_events = []
+
         def control(dirty):
-            '''controls that everything was ok'''
+            """controls that everything was ok"""
             if dirty:
                 self.eq.freeze_rollback()
                 raise ScanTransactionDirty("dirty!")
-            else:
-                return to_later
+
+            # delete metadata for the filtered delete_events
+            fsm = self.fsm
+            for evtname, path in delete_events:
+                parentpath = os.path.dirname(path)
+                log_info("UDF mode! Resetting hashes to dir %r", parentpath)
+                self.fsm.set_by_path(parentpath, local_hash="", server_hash="")
+                if evtname == "FS_DIR_DELETE":
+                    log_info("UDF mode! Removing metadata from dir %r", path)
+                    tree = fsm.get_paths_starting_with(path, include_base=True)
+                    for p, is_dir in tree:
+                        fsm.delete_metadata(p)
+                elif evtname == "FS_FILE_DELETE":
+                    log_info("UDF mode! Removing metadata from file %r", path)
+                    fsm.delete_metadata(path)
+                else:
+                    raise ValueError("Bad delete event! got %s (on %r)"
+                                     % (evtname, path))
+
+            return to_later
+
+        def filter_delete_events(events):
+            """Separate the delete events if it was an UDF."""
+            for evt in events[:]:
+                if evt[0] in ("FS_DIR_DELETE", "FS_FILE_DELETE"):
+                    events.remove(evt)
+                    delete_events.append(evt)
+            return events
 
         d = defer.execute(scan)
+        if udf_mode:
+            d.addCallback(filter_delete_events)
         d.addCallback(self.eq.freeze_commit)
         d.addCallback(control)
         return d
