@@ -24,3 +24,6 @@ from ubuntuone.syncdaemon.interfaces import IMarker
 class MDMarker(str):
     """A marker that has the mdid inside, for action queue."""
     implements(IMarker)
+
+    def __repr__(self):
+        return "marker:%s" % self
