@@ -242,7 +242,7 @@ class FileLogger(object):
     def log(self, lvl, message, *args, **kwargs):
         """Log."""
         format = "%(hasmd)s:%(changed)s:%(isdir)s %(mdid)s "\
-                 "[%(share_id)s:%(node_id)s] '%(path)r' | %(message)s"
+                 "[%(share_id)r::%(node_id)r] '%(path)r' | %(message)s"
         exc_info = sys.exc_info
         if self.key.has_metadata() == "T":
             # catch all errors as we are logging, pylint: disable-msg=W0703
@@ -320,8 +320,7 @@ class SyncStateMachineRunner(StateMachineRunner):
         newstat = os.stat(path)
         if newstat.st_ino != oldstat.st_ino or \
            newstat.st_size != oldstat.st_size or \
-           newstat.st_mtime != oldstat.st_mtime or \
-           newstat.st_ctime != oldstat.st_ctime:
+           newstat.st_mtime != oldstat.st_mtime:
             m = "The received information is obsolete! New stat: %s"
             self.log.debug(m, newstat)
             return False
@@ -394,14 +393,20 @@ class SyncStateMachineRunner(StateMachineRunner):
             #self.m.lr.scan_dir(path)
 
 
-    def new_dir_on_server_with_local(self, event, params, share_id,
-                                     node_id, parent_id, name):
-        """new dir on server and we have a local file."""
+    def new_dir_on_server_with_local_file(self, event, params, share_id,
+                                          node_id, parent_id, name):
+        """New dir on server and we have local file."""
         self.key.move_to_conflict()
         self.key.delete_metadata()
         self.new_dir(event, params, share_id, node_id, parent_id, name)
 
-    def reget_dir(self, event, params, hash):
+    def new_dir_on_server_with_local_dir(self, event, params, share_id,
+                                         node_id, parent_id, name):
+        """New dir on server and we have local dir: re-get it to converge."""
+        self.m.fs.set_node_id(self.key['path'], node_id)
+        self.reget_dir(event, params)
+
+    def reget_dir(self, event, params, hash=None):
         """Reget the directory."""
         self.m.action_q.cancel_download(share_id=self.key['share_id'],
                             node_id=self.key['node_id'])
@@ -440,8 +445,7 @@ class SyncStateMachineRunner(StateMachineRunner):
         """Merge the server directory with the local one."""
         new_files = []
         new_dirs = []
-        deleted_files = []
-        deleted_dirs = []
+        deleted_filedirs = []
         moved = set()
 
         try:
@@ -499,10 +503,7 @@ class SyncStateMachineRunner(StateMachineRunner):
                 if uuid in moved:
                     # this was a move, dont delete
                     continue
-                if isdir:
-                    deleted_dirs.append((share, uuid))
-                else:
-                    deleted_files.append((share, uuid))
+                deleted_filedirs.append((share, uuid))
 
         # these nodes are in process of being deleted from the server, so they
         # are not exactly new locally (were already deleted here)
@@ -510,6 +511,9 @@ class SyncStateMachineRunner(StateMachineRunner):
                                         in self.m.fs.get_iter_trash())
 
         parent_uuid = self.key['node_id']
+        for share, uuid in deleted_filedirs:
+            self.m.event_q.push("SV_FILE_DELETED",
+                                node_id=uuid, share_id=share)
         for share, uuid, name in new_files:
             if (share, uuid) not in trash:
                 self.m.event_q.push("SV_FILE_NEW", parent_id=parent_uuid,
@@ -518,12 +522,6 @@ class SyncStateMachineRunner(StateMachineRunner):
             if (share, uuid) not in trash:
                 self.m.event_q.push("SV_DIR_NEW", parent_id=parent_uuid,
                                     node_id=uuid, share_id=share, name=name)
-        for share, uuid in deleted_files:
-            self.m.event_q.push("SV_FILE_DELETED",
-                                node_id=uuid, share_id=share)
-        for share, uuid in deleted_dirs:
-            self.m.event_q.push("SV_FILE_DELETED",
-                                node_id=uuid, share_id=share)
 
         self.key.remove_partial()
         self.key.set(local_hash=hash)
@@ -656,16 +654,20 @@ class SyncStateMachineRunner(StateMachineRunner):
         self.m.lr.scan_dir(mdid, path)
 
     def new_local_dir_created(self, event, parms, new_id):
-        """we got the server answer for dir creation"""
+        """Server answered that dir creation was ok."""
         self.m.fs.set_node_id(self.key['path'], new_id)
 
+        # query to get any updates in case dir was already there in server
+        self.m.action_q.query([(self.key['share_id'], self.key['node_id'],
+                                self.key['local_hash'])])
+
     def calculate_hash(self, event, params):
         """calculate the hash of this."""
         self.m.hash_q.insert(self.key['path'], self.key['mdid'])
 
-    def rescan_dir(self, event, parms):
+    def rescan_dir(self, event, parms, udfmode):
         """Starts the scan again on a dir."""
-        self.m.lr.scan_dir(self.key['mdid'], self.key['path'])
+        self.m.lr.scan_dir(self.key['mdid'], self.key['path'], udfmode)
 
     def put_file(self, event, params, hash, crc32, size, stat):
         """upload the file to the server."""
@@ -727,8 +729,7 @@ class SyncStateMachineRunner(StateMachineRunner):
         self.key.remove_partial()
         self.key.upload_finished(hash)
 
-
-    def delete_file(self, event, params):
+    def delete_file(self, event, params, *args, **kwargs):
         """server file was deleted."""
         try:
             self.key.delete_file()
@@ -760,6 +761,19 @@ class SyncStateMachineRunner(StateMachineRunner):
         self.key.move_to_conflict()
         self.key.delete_metadata()
 
+    def dir_not_created_in_server(self, event, params, error):
+        """Re-get the dir if it was because already exist."""
+        if error == "ALREADY_EXISTS":
+            # delete metadata and query the father to converge
+            parent = self.m.fs.get_by_node_id(self.key['share_id'],
+                                              self.key['parent_id'])
+            self.key.delete_metadata()
+            self.m.action_q.query([(parent.share_id,
+                                    parent.node_id, parent.local_hash)])
+        else:
+            self.key.move_to_conflict()
+            self.key.delete_metadata()
+
     def delete_on_server(self, event, params, path):
         """local file was deleted."""
         self.m.action_q.unlink(self.key['share_id'],
@@ -797,7 +811,7 @@ class SyncStateMachineRunner(StateMachineRunner):
         self.key.delete_to_trash()
 
     def remove_trash(self, event, params, share_id, node_id):
-        '''Remove the node from trash.'''
+        """Remove the node from trash."""
         self.key.remove_from_trash(share_id, node_id)
 
     def server_moved(self, event, params, share_id, node_id,
@@ -922,6 +936,13 @@ class Sync(object):
         ssmr = SyncStateMachineRunner(self.fsm, self.m, key, log)
         ssmr.signal_event_with_hash("AQ_DOWNLOAD_CANCELLED", server_hash)
 
+    def handle_AQ_DOWNLOAD_DOES_NOT_EXIST(self, share_id, node_id):
+        """on AQ_DOWNLOAD_DOES_NOT_EXIST."""
+        key = FSKey(self.m.fs, share_id=share_id, node_id=node_id)
+        log = FileLogger(self.logger, key)
+        ssmr = SyncStateMachineRunner(self.fsm, self.m, key, log)
+        ssmr.on_event("AQ_DOWNLOAD_DOES_NOT_EXIST", {}, share_id, node_id)
+
     def handle_FS_FILE_CREATE(self, path):
         """on FS_FILE_CREATE"""
         key = FSKey(self.m.fs, path=path)
@@ -999,12 +1020,12 @@ class Sync(object):
         ssmr = SyncStateMachineRunner(self.fsm, self.m, key, log)
         ssmr.on_event('FS_FILE_CLOSE_WRITE', {})
 
-    def handle_LR_SCAN_ERROR(self, mdid):
+    def handle_LR_SCAN_ERROR(self, mdid, udfmode):
         """on LR_SCAN_ERROR"""
         key = FSKey(self.m.fs, mdid=mdid)
         log = FileLogger(self.logger, key)
         ssmr = SyncStateMachineRunner(self.fsm, self.m, key, log)
-        ssmr.on_event('LR_SCAN_ERROR', {})
+        ssmr.on_event('LR_SCAN_ERROR', {}, udfmode)
 
     def handle_HQ_HASH_ERROR(self, mdid):
         """on HQ_HASH_ERROR"""
