@@ -47,7 +47,7 @@ CONSUMER_KEY = "ubuntuone"
 from oauth.oauth import OAuthConsumer
 from ubuntuone.storageprotocol.client import (
     StorageClientFactory, StorageClient)
-from ubuntuone.storageprotocol import request
+from ubuntuone.storageprotocol import request, volumes
 from ubuntuone.storageprotocol.dircontent_pb2 import \
     DirectoryContent, DIRECTORY
 import uuid
@@ -365,7 +365,7 @@ class Client(object):
                                    lambda f: waiter.wake((None, None, f)))
                 else:
                     waiter.wake((d, None, None))
-            except Exception, e:
+            except Exception:
                 waiter.wake((None, sys.exc_info(), None))
 
         self.reactor.callFromThread(runner)
@@ -487,17 +487,21 @@ class Client(object):
             raise CapabilitiesError(e)
 
     @log_timing
-    def get_root_info(self, share_uuid):
+    def get_root_info(self, volume_uuid):
         """Returns the UUID of the applicable share root."""
-        if share_uuid is None:
+        if volume_uuid is None:
             _get_root = self.factory.current_protocol.get_root
             root = self.defer_from_thread(_get_root)
             return (uuid.UUID(root), True)
         else:
-            str_share_uuid = str(share_uuid)
-            share = self._match_share(lambda s: str(s.id) == str_share_uuid)
-            return (uuid.UUID(str(share.subtree)),
-                    share.access_level == "Modify")
+            str_volume_uuid = str(volume_uuid)
+            volume = self._match_volume(lambda v: \
+                                       str(v.volume_id) == str_volume_uuid)
+            if isinstance(volume, volumes.ShareVolume):
+                modify = volume.access_level == "Modify"
+            if isinstance(volume, volumes.UDFVolume):
+                modify = True
+            return (uuid.UUID(str(volume.node_id)), modify)
 
     @log_timing
     def resolve_path(self, share_uuid, root_uuid, path):
@@ -535,21 +539,22 @@ class Client(object):
         return self.defer_from_thread(_resolve_worker)
 
     @log_timing
-    def find_share(self, share_spec):
+    def find_volume(self, volume_spec):
         """Finds a share matching the given UUID.  Looks at both share UUIDs
         and root node UUIDs."""
-        share = self._match_share(lambda s: str(s.id) == share_spec or \
-                                            str(s.subtree) == share_spec)
-        return uuid.UUID(str(share.id))
+        volume = self._match_volume(lambda s: \
+                                    str(s.volume_id) == volume_spec or \
+                                    str(s.node_id) == volume_spec)
+        return uuid.UUID(str(volume.volume_id))
 
     @log_timing
-    def _match_share(self, predicate):
-        """Finds a share matching the given predicate."""
-        _list_shares = self.factory.current_protocol.list_shares
+    def _match_volume(self, predicate):
+        """Finds a volume matching the given predicate."""
+        _list_shares = self.factory.current_protocol.list_volumes
         r = self.defer_from_thread(_list_shares)
-        for share in r.shares:
-            if predicate(share) and share.direction == "to_me":
-                return share
+        for volume in r.volumes:
+            if predicate(volume):
+                return volume
         raise NoSuchShareError()
 
     @log_timing
