@@ -100,7 +100,7 @@ def do_init(client, share_spec, directory, quiet, subtree_path,
     info = metadata.Metadata()
 
     if share_spec is not None:
-        info.share_uuid = client.find_share(share_spec)
+        info.share_uuid = client.find_volume(share_spec)
     else:
         info.share_uuid = None
 
@@ -224,7 +224,7 @@ def do_diff(client, share_spec, directory, quiet, subtree_path,
             ignore_symlinks=True):
     """Diffs a local directory with the server."""
     if share_spec is not None:
-        share_uuid = client.find_share(share_spec)
+        share_uuid = client.find_volume(share_spec)
     else:
         share_uuid = None
     if subtree_path is None:
@@ -442,7 +442,7 @@ def do_main(argv):
         """Capture the exception from calling func."""
         try:
             func()
-        except Exception, e:
+        except Exception:
             queue.put(sys.exc_info())
         else:
             queue.put(None)
